from torch import nn
import torch
import torch.nn.functional as F

class CLIPCompressorIT(nn.Module):
    def __init__(self, img_emb_size=512, text_emb_size=512):
        super().__init__()
        self.img_compressor = nn.Sequential(
            nn.Linear(img_emb_size, 256),
            nn.ReLU(),
            # nn.Linear(256, 100),
            # nn.ReLU(),
            # nn.Linear(100, 100),
            # nn.ReLU(),
            # nn.Linear(100, 100),
            # nn.ReLU(),
            # nn.Linear(100, 100),
            # nn.ReLU(),
            # nn.Linear(100, 100),
            # nn.ReLU(),
            # nn.Linear(100, 100),
            # nn.ReLU(),
            # nn.Linear(100, 100),
            # nn.ReLU(),
            # nn.Linear(100, 100),
            # nn.ReLU(),
            nn.Linear(256, 50),

        )

        self.text_compressor = nn.Sequential(
            nn.Linear(text_emb_size, 256),
            nn.ReLU(),
            # nn.Linear(256, 200),
            # nn.ReLU(),
            # nn.Linear(200, 200),
            # nn.ReLU(),
            # nn.Linear(200, 200),
            # nn.ReLU(),
            # nn.Linear(200, 200),
            # nn.ReLU(),
            # nn.Linear(200, 200),
            # nn.ReLU(),
            # nn.Linear(200, 200),
            # nn.ReLU(),
            nn.Linear(256, 100)
        )

    def forward(self, img_emb, text_emb):
        img_emb = self.img_compressor(img_emb)
        text_emb = self.text_compressor(text_emb)
        return img_emb, text_emb
    


class ProjectionHead(nn.Module):
    def __init__(self, emb_size=512, out_size=512):
        super().__init__()
        self.projection_head = nn.Sequential(
            nn.Linear(emb_size, out_size),
            # nn.ReLU(),
            # nn.Linear(out_size, out_size),
            # nn.ReLU(),
            # nn.Linear(out_size, out_size),
            # nn.Tanh(),
            # nn.Linear(512, out_size),
            # nn.ReLU(),
            # nn.Linear(out_size, out_size),

        )

    def forward(self, emb):
        x = self.projection_head(emb)

        # x is of shape n, d
        # 0 mean and unit varinace for each dimension
        # x = x - x.mean(dim=1, keepdim=True)
        # x = x / (x.std(dim=1, keepdim=True) + 1e-6)
        return x
    

# class AutoEncoder(nn.Module):
#     def __init__(self, emb_size=512, bottle_neck_size=512):
#         super().__init__()
#         self.encoder = nn.Sequential(
#             nn.Linear(emb_size, 300),
#             nn.Sigmoid(),
#             nn.Linear(300, bottle_neck_size),

#         )

#         self.decoder = nn.Sequential(
#             nn.Linear(bottle_neck_size, 300),
#             nn.Sigmoid(),
#             nn.Linear(300, emb_size),

#         )

#     def forward(self, x):
#         z = self.encoder(x)
#         x_hat = self.decoder(z)
#         return x_hat, z
    
class AutoEncoder(nn.Module):
    def __init__(self, emb_size=512, bottle_neck_size=512):
        super().__init__()
        self.encoder = nn.Sequential(
                    nn.Linear(emb_size, emb_size),
                    nn.Sigmoid(),
                    # nn.Linear(emb_size, emb_size),
                    # nn.Sigmoid(),
                    nn.Linear(emb_size, bottle_neck_size),
            )
        self.decoder = nn.Sequential(
                    nn.Linear(bottle_neck_size, emb_size),
                    nn.Sigmoid(),
                    # nn.Linear(emb_size, emb_size),
                    # nn.Sigmoid(),
                    nn.Linear(emb_size, emb_size)
            )
    
    def forward(self, x):
        x = self.encoder(x)
        x = F.normalize(x)
        x = self.decoder(x)
        x = F.normalize(x)
        return x
    

# Create a two layer VAE
class VariationalAE(nn.Module):
    def __init__(self, emb_size=512, bottle_neck_size=512):
        super().__init__()
        self.encoder = nn.Sequential(
            nn.Linear(emb_size, 300),
            nn.Sigmoid(),
            nn.Linear(300, bottle_neck_size),
        )

        self.decoder = nn.Sequential(
            nn.Linear(bottle_neck_size, 300),
            nn.Sigmoid(),
            nn.Linear(300, emb_size),
        )

        self.mu = nn.Linear(bottle_neck_size, bottle_neck_size)
        self.logvar = nn.Linear(bottle_neck_size, bottle_neck_size)

    def forward(self, x):
        z = self.encoder(x)
        mu = self.mu(z)
        logvar = self.logvar(z)
        std = torch.exp(0.5 * logvar)
        eps = torch.randn_like(std)
        z = eps.mul(std).add_(mu)
        x_hat = self.decoder(z)
        return x_hat, z, mu, logvar
        
    