from tirr.packages.netvlad import pittsburgh
import pickle
import sys
import zlib
from tirr.utils.datasets import DatasetPittsTrain, DatasetPitts, DatasetInShop
import torch
from tirr.nn.models import CLIPCompressorIT, ProjectionHead
from torch.nn import functional as F
import numpy as np
import torch.nn as nn
from tirr.utils import get_embeddings
from tirr.sandbox.sandbox import get_recall

import torch
import torch.nn.functional as F

# tf writer tensorboard
from torch.utils.tensorboard import SummaryWriter
from tqdm import tqdm
from sklearn.decomposition import PCA
from cca_zoo.models import CCA



def eval(eval_data, model):
    (
        dbFeatImg,
        qFeatImg,
        dbFeatText,
        qFeatText,
        gt,
        numDb,
        numQ,
    ) = eval_data
    n = 3

    dbFeatComb = get_embeddings.image_text_combine(
        dbFeatImg, dbFeatText, batch_size=32, model=model
    )
    qFeatComb = get_embeddings.image_text_combine(
        qFeatImg, qFeatText, batch_size=32, model=model
    )
    recall_comb = get_recall(qFeatComb, dbFeatComb, gt, numQ, n=n)

    return recall_comb


def student_to_teacher_regression():
    # Hyperparameters
    batch_size = 1000
    proj_dim = 700

    # Set up tensorboard writer
    writer = SummaryWriter("data/tensorboard/mimic_similarity")

    student_embedding = "dinov2"
    teacher_embedding = "dinob8"
    dataset = DatasetInShop(
        no_training=False,
        embed_type1=student_embedding,
        embed_type2=teacher_embedding,
        projected_dim=proj_dim,
        s2t=True,
    )

    studentPca = PCA(n_components=proj_dim)
    _ = studentPca.fit_transform(dataset.dbFeatImg1)

    TeacherPca = PCA(n_components=proj_dim)
    _ = TeacherPca.fit_transform(dataset.dbFeatImg2)

    # Recall for studenct PCA normalzied
    dbFeatImgPca = studentPca.transform(dataset.dbFeatImg1)
    qFeatImgPca = studentPca.transform(dataset.qFeatImg1)
    # mean center and unit variance dbFeatImgPCA and qFeatImgPca
    dbFeatImgPca = (dbFeatImgPca - np.mean(dbFeatImgPca, axis=0)) / np.std(
        dbFeatImgPca, axis=0
    )
    qFeatImgPca = (qFeatImgPca - np.mean(qFeatImgPca, axis=0)) / np.std(
        qFeatImgPca, axis=0
    )
    recall = get_recall(
        qFeatImgPca, dbFeatImgPca, dataset.gt, dataset.numQ, n=4
    )
    print(
        f"Recall@{4} for {student_embedding} images with PCA (normalized): = {recall}"
    )

    # Recall for teacher PCA normalzied
    dbFeatImgPca = TeacherPca.transform(dataset.dbFeatImg2)
    qFeatImgPca = TeacherPca.transform(dataset.qFeatImg2)
    # mean center and unit variance dbFeatImgPCA and qFeatImgPca
    dbFeatImgPca = (dbFeatImgPca - np.mean(dbFeatImgPca, axis=0)) / np.std(
        dbFeatImgPca, axis=0
    )
    qFeatImgPca = (qFeatImgPca - np.mean(qFeatImgPca, axis=0)) / np.std(
        qFeatImgPca, axis=0
    )
    recall = get_recall(
        qFeatImgPca, dbFeatImgPca, dataset.gt, dataset.numQ, n=4
    )
    print(
        f"Recall@{4} for {teacher_embedding} images with PCA (normalized): = {recall}"
    )

    model = ProjectionHead(emb_size=proj_dim, out_size=proj_dim)
    model = model.cuda()

    dataloder = torch.utils.data.DataLoader(
        dataset,
        batch_size=batch_size,
        shuffle=True,
        num_workers=1,
        pin_memory=True,
        drop_last=False,
    )

    optimizer = torch.optim.Adam(model.parameters(), lr=1e-4)

    best_recall = 0
    for epoch in range(1000):  # best recall 0.334
        epoch_loss = []
        for i, batch_data in tqdm(enumerate(dataloder)):
            optimizer.zero_grad()

            img_emb1, img_emb2 = batch_data

            img_emb1 = studentPca.transform(img_emb1)
            img_emb2 = TeacherPca.transform(img_emb2)

            img_emb1 = torch.tensor(img_emb1, dtype=torch.float32).cuda()
            img_emb2 = torch.tensor(img_emb2, dtype=torch.float32).cuda()

            # Running model
            proj_img_emb = model(img_emb1)

            # l2 loss
            loss = F.mse_loss(proj_img_emb, img_emb2)

            epoch_loss.append(loss.item())

            loss.backward()
            optimizer.step()

        print(f"\n\nEpoch {epoch} Loss: {np.mean(epoch_loss)}")

        # save model to data /nn

        # pca qfeatImg1
        qFeatImgPca = studentPca.transform(dataset.qFeatImg1)
        qFeatImgPca = torch.tensor(qFeatImgPca, dtype=torch.float32).cuda()
        qFeatImgProj = model(qFeatImgPca).detach().cpu().numpy()

        # pca dbFeatImg1
        dbFeatImgPca = studentPca.transform(dataset.dbFeatImg1)
        dbFeatImgPca = torch.tensor(dbFeatImgPca, dtype=torch.float32).cuda()
        dbFeatImgProj = model(dbFeatImgPca).detach().cpu().numpy()

        recall_proj = get_recall(
            qFeatImgProj, dbFeatImgProj, dataset.gt, dataset.numQ, n=4
        )
        print(f"Recall Image Proj: {recall_proj}")
        print(f"Best Recall: {best_recall}")
        print(f"Learning Rate: {optimizer.param_groups[0]['lr']}")

        if recall_proj > best_recall:

            print("\n#### Saving Best Model!")
            print(f"Best Recall: {best_recall} -> {recall_proj}")
            best_recall = recall_proj
            torch.save(model.state_dict(), "data/nn/best_model_prjection.pt")


def main():
    # Hyperparameters
    batch_size = 64
    cca_dim_final = 380  # This is the complete CCA dimension
    proj_dim = 380  # This is the projection dimension we are training on
    num_epochs = 100
    use_cca = False

    # Set up tensorboard writer
    writer = SummaryWriter("data/tensorboard/mimic_similarity")

    student_embedding = "dinov2_small"
    teacher_embedding = "dinob8"
    dataset = DatasetInShop(
        no_training=False,
        embed_type1=student_embedding,
        embed_type2=teacher_embedding,
        cca_dim_final=cca_dim_final,
        s2t=False,
    )

    recall_img = get_recall(
        dataset.qFeatImg1, dataset.dbFeatImg1, dataset.gt, dataset.numQ, n=4
    )
    print(f"Recall Image: {recall_img}")

    recall_img_cca = get_recall(
        dataset.qFeatImgCCA, dataset.dbFeatImgCCA, dataset.gt, dataset.numQ, n=4
    )
    print(f"Recall Image CCA: {recall_img_cca}")

    # recall at img1 PCA and img2 PCA
    recall_img = get_recall(
        dataset.qFeatImg1PCA,
        dataset.dbFeatImg1PCA,
        dataset.gt,
        dataset.numQ,
        n=4,
    )
    print(f"Recall Student Image PCA: {recall_img}")

    recall_img = get_recall(
        dataset.qFeatImg2PCA,
        dataset.dbFeatImg2PCA,
        dataset.gt,
        dataset.numQ,
        n=4,
    )
    print(f"Recall Teacher Image PCA: {recall_img}")

    # At target dimensions
    recall_img_cca = get_recall(
        dataset.qFeatImgCCA[:, :proj_dim],
        dataset.dbFeatImgCCA[:, :proj_dim],
        dataset.gt,
        dataset.numQ,
        n=4,
    )
    print(f"Recall Image CCA @{proj_dim}: {recall_img_cca}")

    recall_img1_pca_proj = get_recall(
        dataset.qFeatImg1PCA[:, :proj_dim],
        dataset.dbFeatImg1PCA[:, :proj_dim],
        dataset.gt,
        dataset.numQ,
        n=4,
    )

    print(f"Recall Student Image PCA @{proj_dim}: {recall_img1_pca_proj}")

    recall_img2_pca_proj = get_recall(
        dataset.qFeatImg2PCA[:, :proj_dim],
        dataset.dbFeatImg2PCA[:, :proj_dim],
        dataset.gt,
        dataset.numQ,
        n=4,
    )
    print(f"Recall Teacher Image PCA @{proj_dim}: {recall_img2_pca_proj}")

    breakpoint()

    model = ProjectionHead(emb_size=cca_dim_final, out_size=proj_dim)
    model = model.cuda()

    dataloder = torch.utils.data.DataLoader(
        dataset,
        batch_size=batch_size,
        shuffle=True,
        num_workers=1,
        pin_memory=True,
        drop_last=False,
    )

    optimizer = torch.optim.Adam(model.parameters(), lr=1e-4)

    best_recall = 0
    for epoch in range(num_epochs):
        epoch_loss = []
        for i, batch_data in tqdm(enumerate(dataloder)):
            optimizer.zero_grad()

            img_emb1, img_emb2, img_emb_cca = batch_data

            img_emb1 = img_emb1.reshape(-1, cca_dim_final)
            img_emb2 = img_emb2.reshape(-1, cca_dim_final)
            img_emb_cca = img_emb_cca.reshape(-1, cca_dim_final)

            if use_cca:
                img_emb_cca = img_emb_cca.cuda()
            else:
                img_emb1 = img_emb1.cuda()
            img_emb2 = img_emb2.cuda()

            # Running model
            if use_cca:
                proj_img_emb = model(img_emb_cca)
            else:
                proj_img_emb = model(img_emb1)

            # sim_matrix_cca = F.cosine_similarity(
            #     img_emb_cca.unsqueeze(0), img_emb_cca.unsqueeze(1), dim=2
            # )
            sim_matrix_teacher = F.cosine_similarity(
                img_emb2.unsqueeze(0), img_emb2.unsqueeze(1), dim=2
            )
            prdicted_sim_matrix = F.cosine_similarity(
                proj_img_emb.unsqueeze(0), proj_img_emb.unsqueeze(1), dim=2
            )

            # l2 loss
            loss = F.mse_loss(prdicted_sim_matrix, sim_matrix_teacher)
            # loss = (sim_matrix_cca - prdicted_sim_matrix)**2
            # loss = loss.mean()
            epoch_loss.append(loss.item())
            # loss = triplet_loss(query_img_emb, pos_img_emb, neg_img_emb) # Just for sanity

            loss.backward()
            optimizer.step()

        print(f"\n\nEpoch {epoch} Loss: {np.mean(epoch_loss)}")

        # save model to data /nn

        if use_cca:
            qFeatImgProj = (
                model(torch.tensor(dataset.qFeatImgCCA).cuda())
                .detach()
                .cpu()
                .numpy()
            )
            dbFeatImgProj = (
                model(torch.tensor(dataset.dbFeatImgCCA).cuda())
                .detach()
                .cpu()
                .numpy()
            )

        else:
            qFeatImgProj = (
                model(torch.tensor(dataset.qFeatImg1PCA).cuda())
                .detach()
                .cpu()
                .numpy()
            )
            dbFeatImgProj = (
                model(torch.tensor(dataset.dbFeatImg1PCA).cuda())
                .detach()
                .cpu()
                .numpy()
            )

        recall_proj = get_recall(
            qFeatImgProj, dbFeatImgProj, dataset.gt, dataset.numQ, n=4
        )

        print(f"Recall Image Proj: {recall_img1_pca_proj}")
        print(f"Best Recall: {best_recall}")
        print(f"Current Recall: {recall_proj}")
        print(f"Learning Rate: {optimizer.param_groups[0]['lr']}")

        if recall_proj > best_recall:

            print("\n#### Saving Best Model!")
            print(f"Best Recall: {best_recall} -> {recall_proj}")
            best_recall = recall_proj
            torch.save(model.state_dict(), "data/nn/best_model_prjection.pt")


def metryushka():
    # Hyperparameters
    batch_size = 64
    cca_dim_final = 380  # This is the complete CCA dimension
    proj_dims = [50, 100, 200, 380]  # This is the projection dimension we are training on
    dims_weights = np.array([1,1,1,1], dtype=np.float32)
    num_epochs = 100
    use_cca = True

    # Set up tensorboard writer
    writer = SummaryWriter("data/tensorboard/mimic_similarity")

    student_embedding = "dinov2_small"
    teacher_embedding = "dinob8"
    dataset = DatasetInShop(
        no_training=False,
        embed_type1=student_embedding,
        embed_type2=teacher_embedding,
        cca_dim_final=cca_dim_final,
        s2t=False,
    )

    recall_img = get_recall(
        dataset.qFeatImg1, dataset.dbFeatImg1, dataset.gt, dataset.numQ, n=4
    )

    print(f"Recall Image: {recall_img}")

    recall_img_cca = get_recall(
        dataset.qFeatImgCCA, dataset.dbFeatImgCCA, dataset.gt, dataset.numQ, n=4
    )
    print(f"Recall Image CCA: {recall_img_cca}")

    # recall at img1 PCA and img2 PCA
    recall_img = get_recall(
        dataset.qFeatImg1PCA,
        dataset.dbFeatImg1PCA,
        dataset.gt,
        dataset.numQ,
        n=4,
    )
    print(f"Recall Student Image PCA: {recall_img}")

    recall_img = get_recall(
        dataset.qFeatImg2PCA,
        dataset.dbFeatImg2PCA,
        dataset.gt,
        dataset.numQ,
        n=4,
    )
    print(f"Recall Teacher Image PCA: {recall_img}")


    model = ProjectionHead(emb_size=cca_dim_final, out_size=proj_dims[-1])
    model = model.cuda()

    dataloder = torch.utils.data.DataLoader(
        dataset,
        batch_size=batch_size,
        shuffle=True,
        num_workers=1,
        pin_memory=True,
        drop_last=False,
    )

    optimizer = torch.optim.Adam(model.parameters(), lr=1e-4)

    best_recall = 0
    for epoch in range(num_epochs):
        epoch_loss = []
        for i, batch_data in enumerate(tqdm(dataloder)):
            optimizer.zero_grad()

            img_emb1, img_emb2, img_emb_cca = batch_data

            img_emb1 = img_emb1.reshape(-1, cca_dim_final)
            img_emb2 = img_emb2.reshape(-1, cca_dim_final)
            img_emb_cca = img_emb_cca.reshape(-1, cca_dim_final)

            if use_cca:
                img_emb_cca = img_emb_cca.cuda()
            else:
                img_emb1 = img_emb1.cuda()
            img_emb2 = img_emb2.cuda()

            # Running model
            if use_cca:
                proj_img_emb = model(img_emb_cca)
            else:
                proj_img_emb = model(img_emb1)

            
            sim_matrix_teacher = F.cosine_similarity(
                    img_emb2.unsqueeze(0), img_emb2.unsqueeze(1), dim=2
                )

            # l2 loss
            total_loss = 0
            for dim_idx, proj_dim in enumerate(proj_dims):
                
                if dims_weights[dim_idx] !=0:
                    prdicted_sim_matrix = F.cosine_similarity(
                    proj_img_emb[:, :proj_dim].unsqueeze(0), proj_img_emb[:, :proj_dim].unsqueeze(1), dim=2
                    )


                    loss = F.mse_loss(prdicted_sim_matrix, sim_matrix_teacher) * dims_weights[dim_idx]
                    

                    total_loss +=  loss


            epoch_loss.append(total_loss.item())


            total_loss.backward()

            optimizer.step()



        print(f"\n\nEpoch {epoch} Loss: {np.mean(epoch_loss)}")
        # save model to data /nn
        if use_cca:
            qFeatImgProj = (
                model(torch.tensor(dataset.qFeatImgCCA).cuda())
                .detach()
                .cpu()
                .numpy()
            )
            dbFeatImgProj = (
                model(torch.tensor(dataset.dbFeatImgCCA).cuda())
                .detach()
                .cpu()
                .numpy()
            )

        else:
            qFeatImgProj = (
                model(torch.tensor(dataset.qFeatImg1PCA).cuda())
                .detach()
                .cpu()
                .numpy()
            )
            dbFeatImgProj = (
                model(torch.tensor(dataset.dbFeatImg1PCA).cuda())
                .detach()
                .cpu()
                .numpy()
            )

        if epoch % 10 == 0:
            print('\nRunning Inference:')
            recall_proj_total = 0
            for dim_idx, proj_dim in enumerate(proj_dims):
                # if dims_weights[dim_idx] == 0:
                #     continue
                recall_proj = get_recall(
                    qFeatImgProj[:, :proj_dim].copy(), dbFeatImgProj[:, :proj_dim].copy(), dataset.gt, dataset.numQ, n=4
                )
                print(f"Recall Image Proj @ {proj_dim}: {recall_proj}")
                recall_proj_total += recall_proj * dims_weights[dim_idx]

            # Do CCA of the projected embeddings and dataset.dbFeatImg2PCA 
            imgCCA = CCA(latent_dims=cca_dim_final)
            x = dbFeatImgProj - np.mean(dbFeatImgProj, axis=0)
            y = dataset.dbFeatImg2 - np.mean(dataset.dbFeatImg2, axis=0)
            imgCCA.fit((x, y))

            x = qFeatImgProj - np.mean(qFeatImgProj, axis=0)
            y = dataset.qFeatImg2 - np.mean(dataset.qFeatImg2, axis=0)
            qFeatImgCCA, _ = imgCCA.transform(
                (x, y)
            )

            x = dbFeatImgProj - np.mean(dbFeatImgProj, axis=0)
            y = dataset.dbFeatImg2 - np.mean(dataset.dbFeatImg2, axis=0)
            dbFeatImgCCA, _ = imgCCA.transform(
                (x, y)
            )

            for dim_idx, proj_dim in enumerate(proj_dims):
                # if dims_weights[dim_idx] == 0:
                #     continue

                recall_proj = get_recall(
                    qFeatImgCCA[:, :proj_dim].copy(), dbFeatImgCCA[:, :proj_dim].copy(), dataset.gt, dataset.numQ, n=4
                )
                print(f"CCA Recall Image Proj @ {proj_dim}: {recall_proj}")
                recall_proj_total += recall_proj * dims_weights[dim_idx]


        
        # recall_proj = recall_proj_total / np.sum(dims_weights)

        # print(f"Best Recall: {best_recall}")
        # print(f"Current Recall: {recall_proj}")
        # print(f"Learning Rate: {optimizer.param_groups[0]['lr']}")

        # if recall_proj > best_recall:
        #     print("\n#### Saving Best Model!")
        #     print(f"Best Recall: {best_recall} -> {recall_proj}")
        #     best_recall = recall_proj
        #     torch.save(model.state_dict(), "data/nn/best_model_prjection.pt")


if __name__ == "__main__":

    # student_to_teacher_regression()
    # main()
    metryushka()


"""
50 -> 0.213, 
200 -> 0.314, 0.002, 


Recall Image: 0.1369566746377831
Recall Image CCA: 0.2562948375298917
Recall Student Image PCA: 0.2549409199606133
Recall Teacher Image PCA: 0.3503305668870446
Recall Image CCA @150: 0.24166549444366298    SSD 0.244
Recall Student Image PCA @150: 0.2256998171332114  SSD 0.244
Recall Teacher Image PCA @150: 0.31001195667463777

Epoch Loss: 0.0110


@100
Recall Image CCA @150: 0.223    SSD 0.221      SSD 0.233
Recall Student Image PCA @150: 0.2045  SSD 0.216  SSD 0.230


@50
Recall Image CCA @50: 0._  SSD 0.1901

PCA SSD  M
Recall Image Proj @ 50: 0.19927204951469968
Recall Image Proj @ 200: 0.22682515121676747
Recall Image Proj @ 300: 0.2354585736390491
Recall Image Proj @ 380: 0.24003024335349557

CCA SSD M
Recall Image Proj @ 50: 0.19566746377830918
Recall Image Proj @ 200: 0.22170839780559853
Recall Image Proj @ 300: 0.23148473765649177
Recall Image Proj @ 380: 0.23635532423688282



PCA Metryushka
Recall Image Proj @ 50: 0.19654663103108735
Recall Image Proj @ 100: 0.23974890983260655
Recall Image Proj @ 200: 0.25805317203544803
Recall Image Proj @ 380: 0.26269517513011675


CCA Metryushka
Recall Image Proj @ 50: 0.19457729638486426
Recall Image Proj @ 100: 0.23976649317766213
Recall Image Proj @ 200: 0.25866858911239277
Recall Image Proj @ 380: 0.2624314249542833
"""
