from tirr.utils import singletons
from omegaconf import OmegaConf
import pickle
from tirr.utils import get_embeddings, get_predictions
from tirr.utils.datasets import *
import numpy as np
from sklearn.decomposition import PCA

# import cca
from sklearn.cross_decomposition import CCA as skCCA
from contrastive import CPCA
import pickle
from tqdm import tqdm
from tirr.utils.misc import CustomPCA, CustomCPCA
from tirr.utils.plotting import plot_reduced_emb_exp, plot_reduced_emb_exp_mem, plot_CCA
from torch.nn import functional as F
import torch
from tirr.utils.quantization import (
    binary_quantization,
    quantize_tensor,
    dequantize_tensor,
)
from cca_zoo.models import CCA, GCCA
from cca_zoo.deepmodels import DCCA
from cca_zoo.models import KCCA
from tirr.packages.netvlad import pittsburgh, tokyo247
import torch
from mvlearn.embed import DCCA
from itertools import permutations 
from matplotlib import pyplot as plt
from swarm_visualizer.utility.general_utils import set_plot_properties
from sklearn.manifold import TSNE
from sklearn.metrics.pairwise import cosine_similarity
from sklearn.model_selection import train_test_split
from sklearn.svm import SVC, LinearSVC




def main():
    inshopdir  = '/store/ /datasets/InShop/img/'
    class_label = -1
    instance_label = -1

    data = []

    mendir = inshopdir + 'MEN/'
    womendir = inshopdir + 'WOMEN/'
    for classdir in os.listdir(mendir):
        if classdir[0] == '.':
            continue
        class_label += 1
        for instancedir in os.listdir(mendir + classdir):
            if instancedir[0] == '.':
                continue
            instance_label += 1
            for img in os.listdir(mendir + classdir + '/' + instancedir):
                data.append([mendir + classdir + '/' + instancedir + '/' + img, class_label, instance_label])


    for classdir in os.listdir(womendir):
        if classdir[0] == '.':
            continue
        class_label += 1
        for instancedir in os.listdir(womendir + classdir):
            if instancedir[0] == '.':
                continue
            instance_label += 1
            for img in os.listdir(womendir + classdir + '/' + instancedir):
                data.append([womendir + classdir + '/' + instancedir + '/' + img, class_label, instance_label])

    data = np.array(data)

    all_img_files = data[:, 0]
    dino_emb = get_embeddings.dino(all_img_files, batch_size=72)
    dinov2_emb = get_embeddings.dinov2(all_img_files, batch_size=72)

    # Save embeddings
    with open('data/classification/dino_emb.pkl', 'wb') as f:
        pickle.dump(dino_emb, f)

    with open('data/classification/dinov2_emb.pkl', 'wb') as f:
        pickle.dump(dinov2_emb, f)

    
    breakpoint()    

def main2():
    dataset = DatasetInShop()

    (
        dbFeatImg,
        qFeatImg,
        dbFeatText,
        qFeatText,
        gt,
        numDb,
        numQ,
    ) = dataset.get_dataset_embeddings(embed_type='dino')

    (
        dbFeatImg2,
        qFeatImg2,
        dbFeatText2,
        qFeatText2,
        gt2,
        numDb2,
        numQ2,
    ) = dataset.get_dataset_embeddings(embed_type='dinov2')

    allImgFeat = np.vstack((dbFeatImg, qFeatImg))
    allImgFeat2 = np.vstack((dbFeatImg2, qFeatImg2))
    all_img_files = dataset.all_img_files
    class_map = {}
    instance_map = {}

    all_img_classes = []
    all_img_instances = []

    for img_file in all_img_files:
        img_file_components = img_file.split('/')
        class_name = img_file_components[-4]+ '/' + img_file_components[-3]
        instance_name = class_name + '/' + img_file_components[-2]
        class_map[class_name] = class_map.get(class_name, len(class_map))
        instance_map[instance_name] = instance_map.get(instance_name, len(instance_map))
        all_img_classes.append(class_map[class_name])
        all_img_instances.append(instance_map[instance_name])

    all_img_classes   = np.array(all_img_classes)
    all_img_instances = np.array(all_img_instances)


    def train_and_run_SVM(X, Y):
        X_train, X_test, Y_train, Y_test = train_test_split(X, Y, test_size=0.2, random_state=42)
        clf = LinearSVC()
        clf.fit(X_train, Y_train)

        # print train and test accuracy
        print('Train Accuracy: ', clf.score(X_train, Y_train))
        print('Test Accuracy: ', clf.score(X_test, Y_test))

        # # pring classification report
        # from sklearn.metrics import classification_report
        # y_pred = clf.predict(X_test)
        # print(classification_report(Y_test, y_pred))

    Y = all_img_classes

    for dimension_size in [50, 100, 200]:
        print('\n\n######\nRunning for : ', dimension_size)
        print('Classification Using PCA')
        # pca on all image embeddings
        pca = PCA(n_components=dimension_size)
        pca.fit(allImgFeat)
        X = pca.transform(allImgFeat)
        train_and_run_SVM(X, Y)

        print('Classification Using CCA')
        # CCA on image embeddings
        cca = CCA(latent_dims=dimension_size)
        # normalize
        allImgFeatNorm = allImgFeat - np.mean(allImgFeat, axis=0)
        allImgFeat2Norm = allImgFeat2 - np.mean(allImgFeat2, axis=0)
        cca.fit((allImgFeatNorm, allImgFeat2Norm))

        X, _ = cca.transform((allImgFeatNorm, allImgFeat2Norm))
        
        train_and_run_SVM(X, Y)






if __name__ == "__main__":
    main2()
