from tirr.sandbox.sandbox_imports import *


def pca_ablations(
    k=3, embedding="dino", dataset_name="inshop", metric_name="map", distance="cosine",

):
    # This is a map of dataset classes
    dataset_mapping = get_dataset_mapping()

    # creating an instance of the required dataset class
    dataset = dataset_mapping[dataset_name]()

    metrics_mapping = get_metrics_mapping()

    metric_function = metrics_mapping[metric_name]

    (
        dbFeatImg,
        qFeatImg,
        dbFeatText,
        qFeatText,
        gt,
        numDb,
        numQ,
    ) = dataset.get_dataset_embeddings(embed_type=embedding)

    # Setting up some params
    final_dim_size = dbFeatImg.shape[1]
    comp_range = [0, 1] + list(range(5, final_dim_size-10, 40))
    # comp_range = [2**i for i in range(2, int(np.log2(1500)) + 1)]
    
    # For Tokyo, we have a number of similar images in the db
    num_similar_db = None
    q_in_db = False
    if dataset_name == "tokyo_val":
        q_in_db = True
        num_similar_db = 6

    logger.info("Getting Predictions")

    results = {
        f"pca": [],
        f"pca_drop_dim": [],
    }

    pca = PCA(n_components=comp_range[-1])
    _ = pca.fit_transform(dbFeatImg)
    ##### PCA
    dbFeatImgPca = pca.transform(dbFeatImg)
    qFeatImgPca = pca.transform(qFeatImg)

    # pca eigen values
    eigen_values = pca.explained_variance_ratio_
    breakpoint()

    

   


    for pca_comp in tqdm(comp_range):


        


        metric_value = metric_function(
            qFeatImgPca.copy()[:, pca_comp:],
            dbFeatImgPca.copy()[:, pca_comp:],
            gt,
            numQ,
            k=k,
            q_in_db=q_in_db,
            num_similar_db=num_similar_db,
            distance=distance,
        )
        logger.info(f"{metric_name}@{k} for {embedding} images with PCA: = {metric_value}")

        results[f"pca_drop_dim"].append(metric_value)


    # plot pca_drop_dim
    plt.plot(eigen_values)
    plt.plot(comp_range, results[f"pca_drop_dim"])
    plt.legend(["Eigen Values", "Recall"])
    plt.title("Eigen Values and Recall Compared Against Dimension")

    plt.xlabel("Dimension")
    plt.ylabel("Values")
    plt.savefig("data/plots/maths_pca_drop_dim.png")


    breakpoint()






if __name__ == "__main__":
    cfg = singletons.get_cfg()
    print(
        "\n######\nRunning With The Following Conf:\n", OmegaConf.to_yaml(cfg)
    )
    logger = singletons.get_logger()
    # main()
    k = 4
    embedding = "dinos8"
    dataset_name = "inshop"
    pca_ablations(k=k, embedding=embedding, dataset_name=dataset_name, metric_name="map", distance="cosine")
 



'''
[INFO] | 2024-04-08 16:12:16,442 | <sandbox>  | <pca_ablations> | Embedding Dimension: 768


[INFO] | 2024-04-08 16:12:17,024 | <sandbox>  | <pca_ablations> | recall@1 for dinob8 images with Norm: = 0.6090956887486856


[INFO] | 2024-04-08 16:12:17,514 | <sandbox>  | <pca_ablations> | recall@1 for dinob8 images with PCA: = 0.6021293375394322


[INFO] | 2024-04-08 16:12:18,175 | <sandbox>  | <pca_ablations> | recall@1 for dinob8 images with PCA_Norm_Random: = 0.7147739221871714


[INFO] | 2024-04-08 16:12:18,714 | <sandbox>  | <pca_ablations> | recall@1 for dinob8 images with PCA_Norm: = 0.7147739221871714

8 -> 0.63, 0.41
6 -> 0.62, 0.39
5 -> 0.61, 0.34
4 -> 0.56, 0.21
'''