from tirr.sandbox.sandbox_imports import *


def run_matryoshka(
    embedding="dino",
    dataset_name="inshop",
    metric_name="map",
    distance="cosine",
):
    # Hyperparameters
    batch_size = 64
    proj_dims = [
        16,
        32,
        64,
        128,
        256

    ]  # This is the projection dimension we are training on
    dims_weights = np.array([1, 1, 1, 1, 1], dtype=np.float32)
    num_epochs = 100
    lr = 1e-3

    # Set up tensorboard writer
    writer = SummaryWriter("data/tensorboard/mimic_similarity")

    # This is a map of dataset classes
    dataset_mapping = get_dataset_mapping()

    # creating an instance of the required dataset class
    dataset = dataset_mapping[dataset_name](
        no_training=False, embed_type=embedding
    )


    metrics_mapping = get_metrics_mapping()

    metric_function = metrics_mapping[metric_name]

    recall_img = metric_function(
        dataset.qFeatImg1.copy(),
        dataset.dbFeatImg1.copy(),
        dataset.gt,
        dataset.numQ,
        k=4,
        distance=distance,
    )

    print(f"Recall Image: {recall_img}")

    # recall at img1 PCA and img2 PCA
    recall_img = metric_function(
        dataset.qFeatImg1PCA.copy(),
        dataset.dbFeatImg1PCA.copy(),
        dataset.gt,
        dataset.numQ,
        k=4,
        distance=distance,
    )
    print(f"Recall Student Image PCA: {recall_img}")

    model = ProjectionHead(emb_size=dataset.final_dim_size, out_size=dataset.final_dim_size)
    model = model.cuda()

    dataloder = torch.utils.data.DataLoader(
        dataset,
        batch_size=batch_size,
        shuffle=True,
        num_workers=1,
        pin_memory=True,
        drop_last=False,
    )

    optimizer = torch.optim.AdamW(model.parameters(), lr=lr)

    final_dim = dataset.final_dim_size

    best_recall = 0
    kl_loss = torch.nn.KLDivLoss(reduction="batchmean", log_target=True)
    for epoch in range(num_epochs):
        epoch_loss = []
        for i, batch_data in enumerate(tqdm(dataloder)):
            optimizer.zero_grad()

            img_emb1, sim_values = batch_data
            org_shape = img_emb1.shape

            img_emb1 = img_emb1.reshape(-1, final_dim)

            img_emb1 = img_emb1.cuda()

            proj_img_emb = model(img_emb1)
            proj_img_emb = proj_img_emb.reshape(org_shape[0], org_shape[1], -1)



            

            # sim_matrix_teacher = F.cosine_similarity(
            #     img_emb1.unsqueeze(0), img_emb1.unsqueeze(1), dim=2
            # )

            # l2 loss
            total_loss = 0
            for dim_idx, proj_dim in enumerate(proj_dims):

                if dims_weights[dim_idx] != 0:
                    # prdicted_sim_matrix = F.cosine_similarity(
                    #     proj_img_emb[:, :proj_dim].unsqueeze(0),
                    #     proj_img_emb[:, :proj_dim].unsqueeze(1),
                    #     dim=2,
                    # )

                    proj_img_emb_dim = proj_img_emb[:, :, :proj_dim]
                    # Select embeddings for the first image in each batch
                    first_images = proj_img_emb_dim[:, 0, :]  # Shape: [batch_size, embedding_size]

                    # Reshape to [batch_size, 1, embedding_size] to prepare for broadcasting
                    first_images = first_images.unsqueeze(1)

                    # Reshape proj_img_emb to [batch_size, num_images_per_batch, embedding_size]
                    # Then perform cosine similarity
                    skip_num = 1
                    prdicted_sim_matrix = F.cosine_similarity(first_images, proj_img_emb_dim[:, skip_num:, :], dim=2)

                    # # Calculate softmax for each row of the similarity matrix
                    temperature = 0.7
                    prdicted_sim_matrix_dist = F.log_softmax(prdicted_sim_matrix /temperature, dim=1)
                    sim_values_dist = F.log_softmax(sim_values[:, skip_num:]/temperature, dim=1).cuda()

                    # prdicted_sim_matrix_dist = F.log_softmax(prdicted_sim_matrix_norm /temperature, dim=1)
                    # sim_values_dist = F.log_softmax(sim_values_norm/temperature, dim=1).cuda()

                    # loss_mse = F.mse_loss(prdicted_sim_matrix, sim_values.cuda()) * dims_weights[dim_idx]



                    loss_kl = kl_loss(prdicted_sim_matrix_dist, sim_values_dist) * dims_weights[dim_idx]
                    # print(loss_kl.item(), loss_mse.item())
                    # total_loss += (loss_mse + loss_kl)
                    total_loss += loss_kl
                    # total_loss += loss_mse 

            epoch_loss.append(total_loss.item())

            total_loss.backward()

            optimizer.step()

        print(f"\n\nEpoch {epoch} Loss: {np.mean(epoch_loss)}")

        qFeatImgProj = (
            model(torch.tensor(dataset.qFeatImg1PCA).cuda())
            .detach()
            .cpu()
            .numpy()
        )
        dbFeatImgProj = (
            model(torch.tensor(dataset.dbFeatImg1PCA).cuda())
            .detach()
            .cpu()
            .numpy()
        )

        if epoch % 5 == 0:
            print("\nRunning Inference:")
            recall_proj_total = 0
            for dim_idx, proj_dim in enumerate(proj_dims):
                # if dims_weights[dim_idx] == 0:
                #     continue
                recall_proj = metric_function(
                    qFeatImgProj[:, :proj_dim].copy(),
                    dbFeatImgProj[:, :proj_dim].copy(),
                    dataset.gt,
                    dataset.numQ,
                    k=4,
                    distance=distance,
                )
                print(f"Recall Image Proj @ {proj_dim}: {recall_proj}")
                recall_proj_total += recall_proj * dims_weights[dim_idx]

                # recall at img1 PCA and img2 PCA
                recall_img = metric_function(
                    dataset.qFeatImg1PCA[:, :proj_dim].copy(),
                    dataset.dbFeatImg1PCA[:, :proj_dim].copy(),
                    dataset.gt,
                    dataset.numQ,
                    k=4,
                    distance=distance,
                )
                print(f"Recall Image Va-PCA @ {proj_dim}: {recall_img}")

                # recall at img1 PCA and img2 PCA
                recall_img = metric_function(
                    dataset.qFeatImg1[:, :proj_dim].copy(),
                    dataset.dbFeatImg1[:, :proj_dim].copy(),
                    dataset.gt,
                    dataset.numQ,
                    k=4,
                    distance=distance,
                )
                print(f"Recall Image PCA @ {proj_dim}: {recall_img}")


            torch.save(model.state_dict(), f"data/models/mat_{dataset_name}_{embedding}.pth")



def run_matryoshka_single(
    embedding="dino",
    dataset_name="inshop",
    metric_name="map",
    distance="cosine",
    proj_dim = 32
):
    # Hyperparameters
    batch_size = 64
    proj_dims = [
        proj_dim
    ]  # This is the projection dimension we are training on
    dims_weights = np.array([1], dtype=np.float32)
    num_epochs = 51
    lr = 1e-3

    # Set up tensorboard writer
    writer = SummaryWriter("data/tensorboard/mimic_similarity")

    # This is a map of dataset classes
    dataset_mapping = get_dataset_mapping()

    # creating an instance of the required dataset class
    dataset = dataset_mapping[dataset_name](
        no_training=False, embed_type=embedding
    )


    metrics_mapping = get_metrics_mapping()

    metric_function = metrics_mapping[metric_name]

    recall_img = metric_function(
        dataset.qFeatImg1.copy(),
        dataset.dbFeatImg1.copy(),
        dataset.gt,
        dataset.numQ,
        k=4,
        distance=distance,
    )

    print(f"Recall Image: {recall_img}")

    # recall at img1 PCA and img2 PCA
    recall_img = metric_function(
        dataset.qFeatImg1PCA.copy(),
        dataset.dbFeatImg1PCA.copy(),
        dataset.gt,
        dataset.numQ,
        k=4,
        distance=distance,
    )
    print(f"Recall Student Image PCA: {recall_img}")

    model = ProjectionHead(emb_size=dataset.final_dim_size, out_size=dataset.final_dim_size)
    model = model.cuda()

    dataloder = torch.utils.data.DataLoader(
        dataset,
        batch_size=batch_size,
        shuffle=True,
        num_workers=1,
        pin_memory=True,
        drop_last=False,
    )

    optimizer = torch.optim.AdamW(model.parameters(), lr=lr)

    final_dim = dataset.final_dim_size

    best_recall = 0
    kl_loss = torch.nn.KLDivLoss(reduction="batchmean", log_target=True)
    for epoch in range(num_epochs):
        epoch_loss = []
        for i, batch_data in enumerate(tqdm(dataloder)):
            optimizer.zero_grad()

            img_emb1, sim_values = batch_data
            org_shape = img_emb1.shape

            img_emb1 = img_emb1.reshape(-1, final_dim)

            img_emb1 = img_emb1.cuda()

            proj_img_emb = model(img_emb1)
            proj_img_emb = proj_img_emb.reshape(org_shape[0], org_shape[1], -1)



            

            # sim_matrix_teacher = F.cosine_similarity(
            #     img_emb1.unsqueeze(0), img_emb1.unsqueeze(1), dim=2
            # )

            # l2 loss
            total_loss = 0
            for dim_idx, proj_dim in enumerate(proj_dims):

                if dims_weights[dim_idx] != 0:
                    # prdicted_sim_matrix = F.cosine_similarity(
                    #     proj_img_emb[:, :proj_dim].unsqueeze(0),
                    #     proj_img_emb[:, :proj_dim].unsqueeze(1),
                    #     dim=2,
                    # )

                    proj_img_emb_dim = proj_img_emb[:, :, :proj_dim]
                    # Select embeddings for the first image in each batch
                    first_images = proj_img_emb_dim[:, 0, :]  # Shape: [batch_size, embedding_size]

                    # Reshape to [batch_size, 1, embedding_size] to prepare for broadcasting
                    first_images = first_images.unsqueeze(1)

                    # Reshape proj_img_emb to [batch_size, num_images_per_batch, embedding_size]
                    # Then perform cosine similarity
                    skip_num = 1
                    prdicted_sim_matrix = F.cosine_similarity(first_images, proj_img_emb_dim[:, skip_num:, :], dim=2)

                    # # Calculate softmax for each row of the similarity matrix
                    temperature = 0.7
                    prdicted_sim_matrix_dist = F.log_softmax(prdicted_sim_matrix /temperature, dim=1)
                    sim_values_dist = F.log_softmax(sim_values[:, skip_num:]/temperature, dim=1).cuda()

                    # prdicted_sim_matrix_dist = F.log_softmax(prdicted_sim_matrix_norm /temperature, dim=1)
                    # sim_values_dist = F.log_softmax(sim_values_norm/temperature, dim=1).cuda()

                    # loss_mse = F.mse_loss(prdicted_sim_matrix, sim_values.cuda()) * dims_weights[dim_idx]



                    loss_kl = kl_loss(prdicted_sim_matrix_dist, sim_values_dist) * dims_weights[dim_idx]
                    # print(loss_kl.item(), loss_mse.item())
                    # total_loss += (loss_mse + loss_kl)
                    total_loss += loss_kl
                    # total_loss += loss_mse 

            epoch_loss.append(total_loss.item())

            total_loss.backward()

            optimizer.step()

        print(f"\n\nEpoch {epoch} Loss: {np.mean(epoch_loss)}")

        qFeatImgProj = (
            model(torch.tensor(dataset.qFeatImg1PCA).cuda())
            .detach()
            .cpu()
            .numpy()
        )
        dbFeatImgProj = (
            model(torch.tensor(dataset.dbFeatImg1PCA).cuda())
            .detach()
            .cpu()
            .numpy()
        )

        if epoch % 5 == 0:
            print("\nRunning Inference:")
            recall_proj_total = 0
            for dim_idx, proj_dim in enumerate(proj_dims):
                # if dims_weights[dim_idx] == 0:
                #     continue
                recall_proj = metric_function(
                    qFeatImgProj[:, :proj_dim].copy(),
                    dbFeatImgProj[:, :proj_dim].copy(),
                    dataset.gt,
                    dataset.numQ,
                    k=4,
                    distance=distance,
                )
                print(f"Recall Image Proj @ {proj_dim}: {recall_proj}")
                recall_proj_total += recall_proj * dims_weights[dim_idx]
                
                if recall_proj > best_recall:
                    best_recall = recall_proj
                    torch.save(model.state_dict(), f"data/models/mat_single_{dataset_name}_{embedding}_{proj_dim}.pth")

                print(f"Best recall: {best_recall}")



                # recall at img1 PCA and img2 PCA
                recall_img = metric_function(
                    dataset.qFeatImg1PCA[:, :proj_dim].copy(),
                    dataset.dbFeatImg1PCA[:, :proj_dim].copy(),
                    dataset.gt,
                    dataset.numQ,
                    k=4,
                    distance=distance,
                )
                print(f"Recall Image PCA @ {proj_dim}: {recall_img}")




def infer_matryoshka(
    embedding="dino",
    dataset_name="inshop",
    metric_name="map",
    distance="cosine",
):
    # Hyperparameters
    batch_size = 64
    proj_dims = [
        16,
        32,
        64,
        128,
        256

    ]  # This is the projection dimension we are training on
    dims_weights = np.array([1, 1, 1, 1, 1], dtype=np.float32)
    num_epochs = 500
    lr = 1e-3


    # This is a map of dataset classes
    dataset_mapping = get_dataset_mapping()

    # creating an instance of the required dataset class
    dataset = dataset_mapping[dataset_name](
        no_training=False, embed_type=embedding
    )


    metrics_mapping = get_metrics_mapping()

    metric_function = metrics_mapping[metric_name]

    model = ProjectionHead(emb_size=dataset.final_dim_size, out_size=dataset.final_dim_size)

    # load state dict
    model.load_state_dict(torch.load(f"data/models/mat_{dataset_name}_{embedding}.pth"))
    model = model.cuda()

    print("\nRunning Inference:")
    recall_proj_total = 0
    qFeatImgProj = (
            model(torch.tensor(dataset.qFeatImg1PCA).cuda())
            .detach()
            .cpu()
            .numpy()
        )
    dbFeatImgProj = (
        model(torch.tensor(dataset.dbFeatImg1PCA).cuda())
        .detach()
        .cpu()
        .numpy()
    )

    for dim_idx, proj_dim in enumerate(proj_dims):
        # if dims_weights[dim_idx] == 0:
        #     continue
        recall_proj = metric_function(
            qFeatImgProj[:, :proj_dim].copy(),
            dbFeatImgProj[:, :proj_dim].copy(),
            dataset.gt,
            dataset.numQ,
            k=4,
            distance=distance,
        )
        print(f"Recall Image Proj @ {proj_dim}: {recall_proj}")
        recall_proj_total += recall_proj * dims_weights[dim_idx]

        # recall at img1 PCA and img2 PCA
        recall_img = metric_function(
            dataset.qFeatImg1PCA[:, :proj_dim].copy(),
            dataset.dbFeatImg1PCA[:, :proj_dim].copy(),
            dataset.gt,
            dataset.numQ,
            k=4,
            distance=distance,
        )
        print(f"Recall Image PCA @ {proj_dim}: {recall_img}")



def main():
    run_matryoshka(
        embedding="dinov2_large",
        dataset_name="inshop",
        metric_name="map",
        distance="cosine",
    )

    # infer_matryoshka(
    #     embedding="dinov2_large",
    #     dataset_name="inshop",
    #     metric_name="map",
    #     distance="cosine",
    # )

    # run_matryoshka_single(
    #     embedding="dinos8",
    #     dataset_name="pitts",
    #     metric_name="map",
    #     distance="cosine",
    #     proj_dim = 256
    # )


if __name__ == "__main__":
    main()
