from tirr.utils import singletons
from omegaconf import OmegaConf
import pickle
from tirr.utils import get_embeddings, get_predictions
from tirr.utils.datasets import *
import numpy as np
from sklearn.decomposition import PCA
# import cca
from sklearn.cross_decomposition import CCA as skCCA
from contrastive import CPCA
import pickle
from tqdm import tqdm
from tirr.utils.misc import CustomPCA, CustomCPCA
from tirr.utils.plotting import plot_reduced_emb_exp, plot_reduced_emb_exp_mem, plot_CCA
from torch.nn import functional as F
import torch
from tirr.utils.quantization import (
    binary_quantization,
    quantize_tensor,
    dequantize_tensor,
)
# from cca_zoo.models import CCA, GCCA
from cca_zoo.deepmodels import DCCA
from cca_zoo.models import KCCA
from tirr.packages.netvlad import pittsburgh, tokyo247
import torch
from mvlearn.embed import DCCA
from itertools import permutations 
from matplotlib import pyplot as plt
from swarm_visualizer.utility.general_utils import set_plot_properties
from sklearn.manifold import TSNE

from PIL import Image

from transformers import AutoTokenizer, AutoModelForSequenceClassification, AutoImageProcessor, AutoModel
from sentence_transformers import SentenceTransformer, util
from transformers import ViTImageProcessor, ViTModel
from TIRR.tirr.nn.models import CLIPCompressorIT
from swarm_visualizer.utility.general_utils import set_plot_properties



def get_embeddings(imgs, model, processor):
    img = processor(imgs, return_tensors="pt").to('cuda')
    with torch.no_grad():
        img_emb = model(**img).last_hidden_state
        img_emb = img_emb.mean(dim=1)
    return img_emb.detach().cpu().numpy()

def pca_plot(imgs1, imgs2, l1, l2):
    number_of_images = len(imgs1)

    # load dino model
    processor1 = ViTImageProcessor.from_pretrained('facebook/dino-vitb8')
    model1 = ViTModel.from_pretrained('facebook/dino-vitb8')
    model1 = model1.cuda()

    # load dinov2 model
    processor2 = AutoImageProcessor.from_pretrained('facebook/dinov2-giant')
    model2 = AutoModel.from_pretrained('facebook/dinov2-giant')
    model2 = model2.cuda()

    # get img embeddings
    img1_emb1 = get_embeddings(imgs1, model1, processor1)
    img1_emb2 = get_embeddings(imgs1, model2, processor2)
    img2_emb1 = get_embeddings(imgs2, model1, processor1)
    img2_emb2 = get_embeddings(imgs2, model2, processor2)

    img_emb1 = np.concatenate([img1_emb1, img2_emb1], axis=0)
    img_emb2 = np.concatenate([img1_emb2, img2_emb2], axis=0)

    # # pca
    # pca = PCA(n_components=2)
    # img1_emb1_pca = pca.fit_transform(img1_emb1)
    # img1_emb2_pca = pca.fit_transform(img1_emb2) 
    # img2_emb1_pca = pca.fit_transform(img2_emb1)
    # img2_emb2_pca = pca.fit_transform(img2_emb2)

    # cca = skCCA(n_components=2)
    # img1_emb1_cca, img1_emb2_cca = cca.fit_transform(img1_emb1, img1_emb2)
    # img2_emb1_cca, img2_emb2_cca = cca.fit_transform(img2_emb1, img2_emb2)

    # pca
    pca = PCA(n_components=2)
    img1_emb1_pca = pca.fit_transform(img_emb1)[:number_of_images]
    img1_emb2_pca = pca.fit_transform(img_emb2) [:number_of_images]
    img2_emb1_pca = pca.fit_transform(img_emb1)[number_of_images:]
    img2_emb2_pca = pca.fit_transform(img_emb2)[number_of_images:]

    cca = skCCA(n_components=2)
    img_emb1_cca, img_emb2_cca = cca.fit_transform(img_emb1, img_emb2)
    img1_emb1_cca = img_emb1_cca[:number_of_images]
    img1_emb2_cca = img_emb2_cca[:number_of_images]
    img2_emb1_cca = img_emb1_cca[number_of_images:]
    img2_emb2_cca = img_emb2_cca[number_of_images:]

    # plot
    set_plot_properties()
    fig, ax = plt.subplots(2, 2, figsize=(10, 10))
    ax[0, 0].scatter(img1_emb1_pca[:, 0], img1_emb1_pca[:, 1], c='r', label=l1)
    ax[0, 0].scatter(img2_emb1_pca[:, 0], img2_emb1_pca[:, 1], c='b', label=l2)
    ax[0, 0].set_title('PCA Dino')

    ax[0, 1].scatter(img1_emb2_pca[:, 0], img1_emb2_pca[:, 1], c='r', label=l1)
    ax[0, 1].scatter(img2_emb2_pca[:, 0], img2_emb2_pca[:, 1], c='b', label=l2)
    ax[0, 1].set_title('PCA DinoV2')

    ax[1, 0].scatter(img1_emb1_cca[:, 0], img1_emb1_cca[:, 1], c='r', label=l1)
    ax[1, 0].scatter(img2_emb1_cca[:, 0], img2_emb1_cca[:, 1], c='b', label=l2)
    ax[1, 0].set_title('CCA Dino (DinoV2 guided Dino)')

    ax[1, 1].scatter(img1_emb2_cca[:, 0], img1_emb2_cca[:, 1], c='r', label=l1)
    ax[1, 1].scatter(img2_emb2_cca[:, 0], img2_emb2_cca[:, 1], c='b', label=l2)
    ax[1, 1].set_title('CCA DinoV2 (Dino guided DinoV2)')

    handles, labels = ax[-1,-1].get_legend_handles_labels()
    fig.legend(handles, labels, loc='lower center', ncol=2 )

    fig.suptitle('CCA vs PCA on MNIST Context')


    # save plot
    plt.savefig('data/plots/mnist_context.png')
    breakpoint()


def mnist_context_vs():
    datadir = '/store/ /datasets/mnist_context/' 
    datafile = datadir + 'x_test_cifar_blur0.5.npy'
    targetfile = datadir + 'targets_test_cifar_blur0.5.npy'
    
    data = np.load(datafile)
    targets = np.load(targetfile)

    l1 = 1
    l2 = 0
    img1 = data[targets == l1]
    img2 = data[targets == l2]

    number_of_images = 50
    img1 = img1[:number_of_images]
    img2 = img2[:number_of_images]

    # convert numpy image to PIL image
    img1 = [Image.fromarray(img).convert('RGB') for img in img1]
    img2 = [Image.fromarray(img).convert('RGB') for img in img2]

    pca_plot(img1, img2, str(l1), str(l2))


def water_bird_vs():
    datadir = '/store/ /datasets/CUB_200_2011/CUB_200_2011/images/'
    datadir = '/store/ /datasets/Stanford_Online_Products/'


    # l1dir = datadir + '002.Laysan_Albatross/'
    # l2dir = datadir + '003.Sooty_Albatross/'

    l1dir = datadir + 'chair_final/'
    l2dir = datadir + 'bicycle_final/'

    l1images = [l1dir + x for x in os.listdir(l1dir)]
    l2images = [l2dir + x for x in os.listdir(l2dir)]
    
    img1 = [Image.open(img).convert('RGB') for img in l1images]
    img2 = [Image.open(img).convert('RGB') for img in l2images]

    number_of_images = 50
    img1 = img1[:number_of_images]
    img2 = img2[:number_of_images]

    pca_plot(img1, img2, 'Class 1', 'Class 2')



        


if __name__ == "__main__":
    cfg = singletons.get_cfg()
    print(
        "\n######\nRunning With The Following Conf:\n", OmegaConf.to_yaml(cfg)
    )
    logger = singletons.get_logger()

    # mnist_context_vs()  
    water_bird_vs()


