from tirr.sandbox.sandbox_imports import *


def pca_ablations(
    k=3, embedding="dino", dataset_name="inshop", metric_name="map", distance="cosine",
):
    # This is a map of dataset classes
    dataset_mapping = get_dataset_mapping()

    # creating an instance of the required dataset class
    dataset = dataset_mapping[dataset_name]()

    metrics_mapping = get_metrics_mapping()

    metric_function = metrics_mapping[metric_name]

    (
        dbFeatImg,
        qFeatImg,
        dbFeatText,
        qFeatText,
        gt,
        numDb,
        numQ,
    ) = dataset.get_dataset_embeddings(embed_type=embedding)

    # Setting up some params
    final_dim_size = dbFeatImg.shape[1]
    comp_range = [2**i for i in range(2, int(np.log2(final_dim_size)) + 1)] + [final_dim_size]
    # comp_range = [2**i for i in range(2, int(np.log2(1500)) + 1)]
    
    # For Tokyo, we have a number of similar images in the db
    num_similar_db = None
    q_in_db = False
    if dataset_name == "tokyo_val":
        q_in_db = True
        num_similar_db = 6

    logger.info("Getting Predictions")

    results = {
        f"pca": [],
        f"pca_norm": [],
        f"pca_norm_random": [],
        f"norm_random": [],
        "comp_range": comp_range,
    }

    pca = PCA(n_components=comp_range[-1])
    _ = pca.fit_transform(dbFeatImg)

    # Getting the normalized variants of the original embeddings
    dbFeatImgNorm = (dbFeatImg - np.mean(dbFeatImg, axis=0)) / np.std(
        dbFeatImg, axis=0
    )
    qFeatImgNorm = (qFeatImg - np.mean(dbFeatImg, axis=0)) / np.std(
        dbFeatImg, axis=0
    )

    for pca_comp in tqdm(comp_range):
        logger.info(f"Embedding Dimension: {pca_comp}")

        ##### Original Embedding with Norm
        random_indices = np.random.choice(
            np.arange(comp_range[-1]), pca_comp, replace=False
        )
        metric_value = metric_function(
            qFeatImgNorm.copy()[:, random_indices],
            dbFeatImgNorm.copy()[:, random_indices],
            gt,
            numQ,
            k=k,
            q_in_db=q_in_db,
            num_similar_db=num_similar_db,
            distance=distance,
        )

        logger.info(f"{metric_name}@{k} for {embedding} images with Norm: = {metric_value}")
        results[f"norm_random"].append(metric_value)

        ##### PCA
        dbFeatImgPca = pca.transform(dbFeatImg)[:, :pca_comp]
        qFeatImgPca = pca.transform(qFeatImg)[:, :pca_comp]

        # quantize and dequantize the embeddings
        




        metric_value = metric_function(
            qFeatImgPca.copy(),
            dbFeatImgPca.copy(),
            gt,
            numQ,
            k=k,
            q_in_db=q_in_db,
            num_similar_db=num_similar_db,
            distance=distance,
        )
        logger.info(f"{metric_name}@{k} for {embedding} images with PCA: = {metric_value}")

        results[f"pca"].append(metric_value)

        ##### PCA Norm Random
        dbFeatImgPca = pca.transform(dbFeatImg)
        qFeatImgPca = pca.transform(qFeatImg)

        

        dbMean = np.mean(dbFeatImgPca, axis=0)
        dbStd = np.std(dbFeatImgPca, axis=0)

        dbFeatImgPca = (dbFeatImgPca - dbMean) / dbStd
        qFeatImgPca = (qFeatImgPca - dbMean) / dbStd

        

        # select pca_comp random indixes from 0 to final_dim_size
        random_indices = np.random.choice(
            np.arange(comp_range[-1]), pca_comp, replace=False
        )

        dbFeatImgPca = dbFeatImgPca[:, random_indices]
        qFeatImgPca = qFeatImgPca[:, random_indices]

        metric_value = metric_function(
            qFeatImgPca.copy(),
            dbFeatImgPca.copy(),
            gt,
            numQ,
            k=k,
            q_in_db=q_in_db,
            num_similar_db=num_similar_db,
            distance=distance,
        )
        logger.info(
            f"{metric_name}@{k} for {embedding} images with PCA_Norm_Random: = {metric_value}"
        )

        results[f"pca_norm_random"].append(metric_value)

        ##### PCA Norm
        dbFeatImgPca = pca.transform(dbFeatImg)
        qFeatImgPca = pca.transform(qFeatImg)

        dbMean = np.mean(dbFeatImgPca, axis=0)
        dbStd = np.std(dbFeatImgPca, axis=0)

        dbFeatImgPca = (dbFeatImgPca - dbMean) / dbStd
        qFeatImgPca = (qFeatImgPca - dbMean) / dbStd

        



        dbFeatImgPca = dbFeatImgPca[:, :pca_comp]
        qFeatImgPca = qFeatImgPca[:, :pca_comp]
        metric_value = metric_function(
            qFeatImgPca.copy(),
            dbFeatImgPca.copy(),
            gt,
            numQ,
            k=k,
            q_in_db=q_in_db,
            distance=distance,
        )
        logger.info(
            f"{metric_name}@{k} for {embedding} images with PCA_Norm: = {metric_value}"
        )

        results[f"pca_norm"].append(metric_value)

    # save results pickle
    with open(
        f"data/pkl/{dataset_name}_pca_ablations_{metric_name}@{k}_{embedding}.pkl",
        "wb",
    ) as f:
        pickle.dump(results, f)



def main():
    all_embeddings = [
        # "dinov2_vitg14",
        # "dinob8",
        # "dinos8",
        # "dinov2_small",
        # "dinov2_large",
        # "vit",
        # "clip",
        'dinob16'
    ]

    dataset_names = ["pitts", "inshop", "tokyo_val", "sop", ]
    k = 4
    for dataset_name in dataset_names:
        for embedding in all_embeddings:
            pca_ablations(k=k, embedding=embedding, dataset_name=dataset_name, metric_name="map")
            plot_pca_ablations(
                k=k, embedding=embedding, dataset_name=dataset_name, metric_name="map"
            )


if __name__ == "__main__":
    cfg = singletons.get_cfg()
    print(
        "\n######\nRunning With The Following Conf:\n", OmegaConf.to_yaml(cfg)
    )
    logger = singletons.get_logger()
    main()
    # k = 4
    # embedding = "clip"
    # dataset_name = "pitts"
    # pca_ablations(k=k, embedding=embedding, dataset_name=dataset_name, metric_name="map", distance="cosine")
    # plot_pca_ablations(k=k, embedding=embedding, dataset_name=dataset_name, metric_name="map")
