from tirr.sandbox.sandbox_imports import *


def pca_ablations(
    k=3, embedding="dino", dataset_name="inshop", metric_name="map", distance="cosine",
    quantization_level = 6

):
    # This is a map of dataset classes
    dataset_mapping = get_dataset_mapping()

    # creating an instance of the required dataset class
    dataset = dataset_mapping[dataset_name]()

    metrics_mapping = get_metrics_mapping()

    metric_function = metrics_mapping[metric_name]

    (
        dbFeatImg,
        qFeatImg,
        dbFeatText,
        qFeatText,
        gt,
        numDb,
        numQ,
    ) = dataset.get_dataset_embeddings(embed_type=embedding)

    # Setting up some params
    final_dim_size = dbFeatImg.shape[1]
    comp_range = [2**i for i in range(7, int(np.log2(final_dim_size)) + 1)] + [final_dim_size]
    # comp_range = [2**i for i in range(2, int(np.log2(1500)) + 1)]
    
    # For Tokyo, we have a number of similar images in the db
    num_similar_db = None
    q_in_db = False
    if dataset_name == "tokyo_val":
        q_in_db = True
        num_similar_db = 6

    logger.info("Getting Predictions")

    results = {
        f"pca": [],
        f"pca_norm": [],
        f"pca_norm_random": [],
        f"norm_random": [],
        "comp_range": comp_range,
    }

    pca = PCA(n_components=comp_range[-1])
    _ = pca.fit_transform(dbFeatImg)

    # Getting the normalized variants of the original embeddings
    dbFeatImgNorm = (dbFeatImg - np.mean(dbFeatImg, axis=0)) / np.std(
        dbFeatImg, axis=0
    )
    qFeatImgNorm = (qFeatImg - np.mean(dbFeatImg, axis=0)) / np.std(
        dbFeatImg, axis=0
    )

    skip_dims = 80

    for pca_comp in tqdm(comp_range):
        logger.info(f"Embedding Dimension: {pca_comp}")

        ##### Original Embedding with Norm
        random_indices = np.random.choice(
            np.arange(comp_range[-1]), pca_comp, replace=False
        )
        metric_value = metric_function(
            qFeatImgNorm.copy()[:, random_indices],
            dbFeatImgNorm.copy()[:, random_indices],
            gt,
            numQ,
            k=k,
            q_in_db=q_in_db,
            num_similar_db=num_similar_db,
            distance=distance,
        )

        logger.info(f"{metric_name}@{k} for {embedding} images with Norm: = {metric_value}")
        results[f"norm_random"].append(metric_value)

        ##### PCA
        dbFeatImgPca = pca.transform(dbFeatImg)[:, :pca_comp]
        qFeatImgPca = pca.transform(qFeatImg)[:, :pca_comp]

        # quantize and dequantize the embeddings
        dbFeatImgPca = quantize_tensor(
            dbFeatImgPca, num_bits=quantization_level
        )
        dbFeatImgPca = dequantize_tensor(dbFeatImgPca).numpy()
        qFeatImgPca = quantize_tensor(qFeatImgPca, num_bits=quantization_level)
        qFeatImgPca = dequantize_tensor(qFeatImgPca).numpy()




        metric_value = metric_function(
            qFeatImgPca.copy()[:, skip_dims:],
            dbFeatImgPca.copy()[:, skip_dims:],
            gt,
            numQ,
            k=k,
            q_in_db=q_in_db,
            num_similar_db=num_similar_db,
            distance=distance,
        )
        logger.info(f"{metric_name}@{k} for {embedding} images with PCA: = {metric_value}")

        results[f"pca"].append(metric_value)

        ##### PCA Norm Random
        dbFeatImgPca = pca.transform(dbFeatImg)
        qFeatImgPca = pca.transform(qFeatImg)

        

        dbMean = np.mean(dbFeatImgPca, axis=0)
        dbStd = np.std(dbFeatImgPca, axis=0)

        dbFeatImgPca = (dbFeatImgPca - dbMean) / dbStd
        qFeatImgPca = (qFeatImgPca - dbMean) / dbStd

        dbFeatImgPca = quantize_tensor(
            dbFeatImgPca, num_bits=quantization_level
        )
        dbFeatImgPca = dequantize_tensor(dbFeatImgPca).numpy()
        qFeatImgPca = quantize_tensor(qFeatImgPca, num_bits=quantization_level)
        qFeatImgPca = dequantize_tensor(qFeatImgPca).numpy()

        # select pca_comp random indixes from 0 to final_dim_size
        random_indices = np.random.choice(
            np.arange(comp_range[-1]), pca_comp, replace=False
        )

        dbFeatImgPca = dbFeatImgPca[:, random_indices]
        qFeatImgPca = qFeatImgPca[:, random_indices]

        metric_value = metric_function(
            qFeatImgPca.copy(),
            dbFeatImgPca.copy(),
            gt,
            numQ,
            k=k,
            q_in_db=q_in_db,
            num_similar_db=num_similar_db,
            distance=distance,
        )
        logger.info(
            f"{metric_name}@{k} for {embedding} images with PCA_Norm_Random: = {metric_value}"
        )

        results[f"pca_norm_random"].append(metric_value)

        ##### PCA Norm
        dbFeatImgPca = pca.transform(dbFeatImg)
        qFeatImgPca = pca.transform(qFeatImg)

        dbMean = np.mean(dbFeatImgPca, axis=0)
        dbStd = np.std(dbFeatImgPca, axis=0)

        dbFeatImgPca = (dbFeatImgPca - dbMean) / dbStd
        qFeatImgPca = (qFeatImgPca - dbMean) / dbStd

        dbFeatImgPca = quantize_tensor(
            dbFeatImgPca, num_bits=quantization_level
        )
        dbFeatImgPca = dequantize_tensor(dbFeatImgPca).numpy()
        qFeatImgPca = quantize_tensor(qFeatImgPca, num_bits=quantization_level)
        qFeatImgPca = dequantize_tensor(qFeatImgPca).numpy()



        dbFeatImgPca = dbFeatImgPca[:, :pca_comp]
        qFeatImgPca = qFeatImgPca[:, :pca_comp]
        metric_value = metric_function(
            qFeatImgPca.copy()[:, skip_dims:],
            dbFeatImgPca.copy()[:, skip_dims:],
            gt,
            numQ,
            k=k,
            q_in_db=q_in_db,
            distance=distance,
        )
        logger.info(
            f"{metric_name}@{k} for {embedding} images with PCA_Norm: = {metric_value}"
        )

        results[f"pca_norm"].append(metric_value)

    # save results pickle
    with open(
        f"data/pkl/{dataset_name}_pca_ablations_{metric_name}@{k}_{embedding}_{quantization_level}.pkl",
        "wb",
    ) as f:
        pickle.dump(results, f)



def main():
    all_embeddings = [
        "dinov2_vitg14",
        "dinob8",
        "dinos8",
        "dinov2_small",
        "dinov2_large",
        "vit",
        "clip",
    ]

    dataset_names = ["pitts", "inshop", "tokyo_val", "sop", ]
    k = 1
    for dataset_name in dataset_names:
        for embedding in all_embeddings:
            pca_ablations(k=k, embedding=embedding, dataset_name=dataset_name, metric_name="recall")
            plot_pca_ablations(
                k=k, embedding=embedding, dataset_name=dataset_name, metric_name="recall"
            )


if __name__ == "__main__":
    cfg = singletons.get_cfg()
    print(
        "\n######\nRunning With The Following Conf:\n", OmegaConf.to_yaml(cfg)
    )
    logger = singletons.get_logger()
    # main()
    k = 4
    embedding = "dinos8"
    dataset_name = "inshop"
    pca_ablations(k=k, embedding=embedding, dataset_name=dataset_name, metric_name="map", distance="cosine", quantization_level=32)
    plot_pca_ablations(k=k, embedding=embedding, dataset_name=dataset_name, metric_name="map", quantization_level=32)
    # plot_pca_ablations(k=k, embedding=embedding, dataset_name=dataset_name, metric_name="map", quantization_level=6)
    # plot_pca_ablations(k=k, embedding=embedding, dataset_name=dataset_name, metric_name="map", quantization_level=5)
    # plot_pca_ablations(k=k, embedding=embedding, dataset_name=dataset_name, metric_name="map", quantization_level=4)



'''
[INFO] | 2024-04-08 16:12:16,442 | <sandbox>  | <pca_ablations> | Embedding Dimension: 768


[INFO] | 2024-04-08 16:12:17,024 | <sandbox>  | <pca_ablations> | recall@1 for dinob8 images with Norm: = 0.6090956887486856


[INFO] | 2024-04-08 16:12:17,514 | <sandbox>  | <pca_ablations> | recall@1 for dinob8 images with PCA: = 0.6021293375394322


[INFO] | 2024-04-08 16:12:18,175 | <sandbox>  | <pca_ablations> | recall@1 for dinob8 images with PCA_Norm_Random: = 0.7147739221871714


[INFO] | 2024-04-08 16:12:18,714 | <sandbox>  | <pca_ablations> | recall@1 for dinob8 images with PCA_Norm: = 0.7147739221871714

8 -> 0.63, 0.41
6 -> 0.62, 0.39
5 -> 0.61, 0.34
4 -> 0.56, 0.21
'''