import open_clip
import torch
import numpy as np
from PIL import Image
from tqdm import tqdm

class CLIPImg():
    def __init__(self, model_name="ViT-B/32"):
        self.model, _, self.preprocess = open_clip.create_model_and_transforms('ViT-B-32', pretrained='laion2b_s34b_b79k')
        # self.model = self.model.cuda()

    def get_embeddings(self, img_files, batch_size=32):
        img_embeddings = []
        with torch.no_grad(), torch.cuda.amp.autocast():
            for i in range(0, len(img_files), batch_size):
                batch = []
                for img_file in img_files[i:i+batch_size]:
                    image = self.preprocess(Image.open(img_file)).unsqueeze(0)
                    batch.append(image)
                batch = torch.cat(batch, dim=0)
                # batch = batch.cuda()
                image_features = self.model.encode_image(batch)
                image_features /= image_features.norm(dim=-1, keepdim=True)
                img_embeddings.append(image_features.detach().cpu().numpy())
        img_embeddings = np.concatenate(img_embeddings, axis=0)

        return img_embeddings
    
class CLIPText():
    def __init__(self, model_name="ViT-B/32"):
        self.model, _, self.preprocess = open_clip.create_model_and_transforms('ViT-B-32', pretrained='laion2b_s34b_b79k')
        self.tokenizer = open_clip.get_tokenizer('ViT-B-32')
        # self.model = self.model.cuda()

    def get_embeddings(self, text, batch_size=32):
        text_features = []
        with torch.no_grad(), torch.cuda.amp.autocast():
            for i in range(0, len(text), batch_size):
                batch = text[i:i+batch_size]
                batch = self.tokenizer(batch)
                # batch = batch.cuda()
                batch = self.model.encode_text(batch)
                batch /= batch.norm(dim=-1, keepdim=True)
                text_features.append(batch.detach().cpu().numpy())
        text_features = np.concatenate(text_features, axis=0)

        return text_features