import faiss
from sklearn.metrics.pairwise import cosine_similarity
import numpy as np
from tqdm import tqdm

def faiss_preds(qFeat, dbFeat, n, index_type):
    if index_type == 'cosine':
        faiss_index = faiss.IndexFlatIP(qFeat.shape[1])
    elif index_type == 'l2':
        faiss_index = faiss.IndexFlatL2(qFeat.shape[1])
    else:
        print('Invalid index type')
        raise NotImplementedError
    faiss_index.add(dbFeat)

    distances, predictions = faiss_index.search(qFeat, n)
    return distances, predictions


def joint_img_text_preds(qFeatImg, qFeatText, dbFeatImg, dbFeatText, n, gt):
    text_n = max(20, qFeatImg.shape[1])
    faiss_index = faiss.IndexFlatIP(qFeatText.shape[1])
    faiss_index.add(dbFeatText)
    distancesText, predictionsText = faiss_index.search(qFeatText, text_n)

    predictions = []
    for qidx in range(qFeatText.shape[0]):
        topImgs =  dbFeatImg[predictionsText[qidx]]
        sim = cosine_similarity(qFeatImg[qidx].reshape(1, -1), topImgs)[0]
        predictions.append(predictionsText[qidx][np.argsort(sim)[::-1][:n]])

    return None, np.array(predictions)

def joint_img_text_preds_custom(qFeatImg, qFeatText, dbFeatImg, dbFeatText, n, w):
    predictions = []
    for qidx in tqdm(range(qFeatText.shape[0])):
        simImg = cosine_similarity(qFeatImg[qidx].reshape(1, -1), dbFeatImg)[0]
        simText = cosine_similarity(qFeatText[qidx].reshape(1, -1), dbFeatText)[0]
        sim = simImg + w * simText

        predictions.append(np.argsort(sim)[::-1][:n])

    return None, np.array(predictions)
        
