from swarm_visualizer.utility.general_utils import set_plot_properties
from matplotlib import pyplot as plt
import pickle
import numpy as np
import pandas as pd
from math import pi
from tirr.utils import singletons
from swarm_visualizer.boxplot import plot_paired_boxplot
import seaborn as sns
from matplotlib.font_manager import FontProperties


# logger and config
cfg = singletons.get_cfg()
logger = singletons.get_logger()


def plot_compute_vs_performance():
    # Name, Params in Millions, Precision@4
    data = [
        ["Dinov2-Small", 22.1, 0.213],
        ["Dinob8", 85.8, 0.296],
        ["[CCA]: Dinob8 guided Dinov2-Small", 22.1 + 0.14, 0.231],
        # ['CLIP', 2539.56, 0.519],
        ["[CCA]: CLIP guided Dinov2-Small", 22.1 + 0.14, 0.261],
        ["[FULL] Dinov2-Small", 22.1, 0.254],
    ]

    set_plot_properties()

    # scatter plot with params on x-axis and precision on y-axis
    fig, ax = plt.subplots()
    for d in data:
        ax.scatter(d[1], d[2], label=d[0])

    # for d in data:
    #     ax.annotate(d[0], (d[1], d[2]))

    plt.xlabel("Params (Millions)")
    plt.ylabel("Precision@4 for Embedding size 100")
    plt.title("Compute vs Performance")
    plt.legend()

    plt.savefig("data/plots/compute_vs_performance.png")


def plot_pca_ablations(k, embedding, dataset_name, metric_name):
    set_plot_properties()
    plt.figure(figsize=(8, 6))

    plt.rcParams["text.usetex"] = True
    plt.rcParams["text.latex.preamble"] = r"\usepackage{bm}"

    # open pickle file
    with open(
        f"data/pkl/{dataset_name}_pca_ablations_{metric_name}@{k}_{embedding}.pkl",
        "rb",
    ) as f:
        results = pickle.load(f)

    lables = {
        "pca": "PCA",
        "pca_norm": "PCA + Norm (Ours)",
        "pca_norm_random": "PCA + Norm + Random",
        "norm_random": "Norm + Random",
    }

    metric_font = {
        "recall": "Recall",
        "map": "mAP",
    }

    for key in results:
        if key == "comp_range":
            continue
        plt.plot(results["comp_range"], results[key], label=lables[key])

    plt.legend(loc="lower right", fontsize=15)

    plt.title(
        f"PCA Ablations with {embedding.upper()} on {dataset_name.upper()} bit quantization",
        fontsize=20,
    )

    plt.xlabel("Embedding Dimension", fontsize=18)
    plt.ylabel(f"{metric_font[metric_name]}@{k}", fontsize=18)
    plt.savefig(f"data/plots/{dataset_name}_pca_ablations_{metric_name}@{k}_{embedding}.png")
    logger.info(f"Saved plot: data/plots/{dataset_name}_pca_ablations_{metric_name}@{k}_{embedding}.png")
    plt.clf()

def plot_flops_vs_performance():
    # Name, Params in Millions, Precision@4
    data = [
        ["PCA+Norm+SSDwM (ours)", (2 * 32 * 17000 + 256*256*2 + 256*2) / 1000_000, 0.268],
        ["PCA+Norm (ours)", (2 * 32 * 17000 + 256*256 + 256*2)/ 1000_000, 0.208],
        ["PCA+Norm+Random", (2 * 32 * 17000 + 256*256 + 256*2)/ 1000_000, 0.153],
        # ['CLIP', 2539.56, 0.519],
        ["PCA", (2 * 32 * 17000 + 256*256)/ 1000_000, 0.175],
        ["Norm+Random", (2 * 32 * 17000 + 256*2)/ 1000_000, 0.180],
    ]

    colors = {
        "PCA+Norm+SSDwM (ours)": "black",
        "PCA+Norm (ours)": "orange",
        "PCA+Norm+Random": "green",
        "PCA": "blue",
        "Norm+Random": "red",
    }

    set_plot_properties()
    


    # scatter plot with params on x-axis and precision on y-axis
    fig, ax = plt.subplots()
    # set ig size 10,10
    fig.set_size_inches(7, 7)
    
    for d in data:
        ax.scatter(d[1], d[2], label=d[0], color=colors[d[0]])

    # set x axis limits
    plt.xlim(1, 1.5)

    plt.xlabel("GFLOPs")
    plt.ylabel("mAP@4 for Embedding size 32")
    plt.title("Compute vs Performance")
    plt.legend()

    plt.savefig("data/plots/compute_vs_performance.png")


def plot_pca_ablations_with_M(k, embedding, dataset_name, metric_name):
    set_plot_properties()
    plt.figure(figsize=(8, 6))

    plt.rcParams["text.usetex"] = True
    plt.rcParams["text.latex.preamble"] = r"\usepackage{bm}"

    # open pickle file
    with open(
        f"data/pkl/{dataset_name}_pca_ablations_{metric_name}@{k}_{embedding}.pkl",
        "rb",
    ) as f:
        results = pickle.load(f)

    lables = {
        "pca": "PCA",
        "pca_norm": "PCA + Norm (Ours)",
        "pca_norm_random": "PCA + Norm + Random",
        "norm_random": "Norm + Random",
    }

    metric_font = {
        "recall": "Recall",
        "map": "mAP",
    }
    plt.plot([16, 32, 64, 128, 256], [0.235, 0.361, 0.450, 0.522, 0.558], label="PCA+Norm+SSDwM", color="black", alpha=0.5)

    for key in results:
        if key == "comp_range":
            continue
        plt.plot(results["comp_range"], results[key], label=lables[key])

    plt.legend(loc="lower right", fontsize=15)

    plt.title(
        f"PCA Ablations with {embedding.upper()} on {dataset_name.upper()}",
        fontsize=20,
    )

    plt.xlabel("Embedding Dimension", fontsize=18)
    plt.ylabel(f"{metric_font[metric_name]}@{k}", fontsize=18)
    plt.savefig(f"data/plots/mat_{dataset_name}_pca_ablations_{metric_name}@{k}_{embedding}.png")
    plt.clf()


def plot_flops_vs_performance_full(k, embedding, dataset_name, metric_name):
    set_plot_properties()
    plt.figure(figsize=(6, 5))

    plt.rcParams["text.usetex"] = True
    plt.rcParams["text.latex.preamble"] = r"\usepackage{bm}"

    # open pickle file
    with open(
        f"data/pkl/{dataset_name}_pca_ablations_{metric_name}@{k}_{embedding}.pkl",
        "rb",
    ) as f:
        results = pickle.load(f)

    lables = {
        "pca": "PCA",
        "pca_norm": "VA-PCA (Ours)",
        "pca_norm_random": "PCA + Norm + Random",
        "norm_random": "Norm + Random",
    }

    dataset_size = 17000
    final_emb_size = 256


    dataset_size_kb_32 = dataset_size * 32 * cfg['constants']['memory']['single_float64']
    dataset_size_kb_64 = dataset_size * 64 * cfg['constants']['memory']['single_float64']
    dataset_size_kb_128 = dataset_size * 128 * cfg['constants']['memory']['single_float64']

    print(f"Dataset size in MB: {dataset_size_kb_32 / 1000}")
    print(f"Dataset size in MB: {dataset_size_kb_64 / 1000}")
    print(f"Dataset size in MB: {dataset_size_kb_128 / 1000}")

    print("Datasetsize Per Elem: ", dataset_size_kb_32 / dataset_size)
    print("Datasetsize Per Elem: ", dataset_size_kb_64 / dataset_size)
    print("Datasetsize Per Elem: ", dataset_size_kb_128 / dataset_size)

    approach_glop_functions = {
        "pca": lambda x: 2 * x * dataset_size + final_emb_size * final_emb_size + 256 * 2,
        "pca_norm": lambda x: 2 * x * dataset_size + final_emb_size * final_emb_size + 256 * 2,
        "pca_norm_random": lambda x: 2 * x * dataset_size + final_emb_size * final_emb_size + 256 * 2,
        "norm_random": lambda x: 2 * x * dataset_size + final_emb_size * 2,
        "pca_norm_ssdwm": lambda x: 2 * x * dataset_size + final_emb_size * final_emb_size*2 + 256 * 2,
    }

    metric_font = {
        "recall": "Recall",
        "map": "mAP",
    }

    for key in results:
        if key == "comp_range":
            continue
        elif key == "pca" or key == "pca_norm":
            f_glop = approach_glop_functions[key]
            plt.plot([f_glop(x) / 1000_000 for x in results["comp_range"]], results[key], label=lables[key], marker="x")

    ssdwm_comp = [16, 32, 64, 128, 256]
    ssdwm_perf = [0.235, 0.361, 0.450, 0.522, 0.558]
    plt.plot([approach_glop_functions["pca_norm_ssdwm"](x) / 1000_000 for x in ssdwm_comp], ssdwm_perf, label="(SS)2D (Ours)", color="green", alpha=1, marker="x")

    plt.legend(loc="lower right", fontsize=15)

    # plt.title(
    #     f"GFLOPs Comparison with {embedding.upper()} on {dataset_name.upper()}",
    #     fontsize=20,
    # )

    plt.xlabel("GFLOPs", fontsize=18)
    plt.ylabel(f"{metric_font[metric_name]}@{k}", fontsize=18)
    plt.savefig(f"data/plots/perfromance_{dataset_name}_{metric_name}@{k}_{embedding}.png")
    plt.clf()


def plot_va_pca_combined_line(k, embedding_list, dataset_names, metric_name):
    set_plot_properties()
    fig, axs = plt.subplots(1, 4, figsize=(18, 4))  # Create subplots side by side

    plt.rcParams["text.usetex"] = True
    plt.rcParams["text.latex.preamble"] = r"\usepackage{bm}"

    dataset_map = {
        "inshop": "InShop",
        "sop": "SOP",
        "pitts": "Pittsburgh30K",
        "tokyo_val": "TokyoVal",
    }

    embedding_map = {
        "clip": "CLIP",
        "vit": "ViT",
        "dinob16": "DINO-B16",
        "dinov2_large": "DINO-V2-Large",
        "dinos8": "DINO-S8",
    }

    # Lists to store legend handles and labels
    handles = []
    labels = []

    # Loop through each dataset
    for i, dataset_name in enumerate(dataset_names):
        # open pickle file
        all_results = []
        for embedding in embedding_list:
            with open(
                f"data/pkl/{dataset_name}_pca_ablations_{metric_name}@{k}_{embedding}.pkl",
                "rb",
            ) as f:
                results = pickle.load(f)
                all_results.append(results)

        lables = {
            "pca": "PCA",
            "pca_norm": "VA-PCA (Ours)",
            "pca_norm_random": "VA-PCA + Random",
            "norm_random": "Norm + Random",
        }

        metric_font = {
            "recall": "Recall",
            "map": "mAP",
        }

        # different colors for each embedding
        colors_list = {
            'clip': 'red',
            'vit': 'blue',
            'dinob16': 'green',
            'dinov2_large': 'orange',
        }

        for idx in range(len(embedding_list)):
            embedding = embedding_list[idx]
            results = all_results[idx]
            for key in results:
                if key == "pca":
                    axs[i].plot(results["comp_range"], results[key], label=f'{embedding_map[embedding]} with {lables[key]}', color=colors_list[embedding], linestyle="--")
                elif key == "pca_norm":
                    axs[i].plot(results["comp_range"], results[key], label=f'{embedding_map[embedding]} with {lables[key]}', color=colors_list[embedding], linestyle="-")

        # font style bold in title
        axs[i].set_title(f'{dataset_map[dataset_name]}', fontsize=20, fontproperties=FontProperties(weight='bold'))
        axs[i].set_xlim(0, 800)
        axs[i].set_xlabel("Embedding Dimension", fontsize=18)
        axs[i].set_ylabel(f"{metric_font[metric_name]}@{k}", fontsize=18)

        # # Add handles and labels for legends
        if i == 0:
            handles_axs, labels_axs = axs[i].get_legend_handles_labels()
            handles.extend(handles_axs)
            labels.extend(labels_axs)

    # Common legend and title
    # plt.subplots_adjust(bottom=0.4)
    fig.legend(handles, labels, loc='lower center', bbox_to_anchor=(0.5, -0.2), shadow=True, ncol=3, fontsize=18)
    # fig.suptitle("Impact of VA-PCA on Retrieval Performance Across Four Datasets", fontsize=25)

    # Save plot
    plt.savefig(f"data/plots/vapca_{metric_name}@{k}_combined_line.png", bbox_inches='tight')
    logger.info(f"Saved plot: data/plots/vapca_{metric_name}@{k}_combined_line.png")
    plt.clf()


def plot_va_pca_combined_block(k, embedding_list, dataset_names, metric_name):
    set_plot_properties()
    fig, axs = plt.subplots(2, 2, figsize=(10, 9))  # Create subplots in a 2x2 grid

    plt.rcParams["text.usetex"] = True
    plt.rcParams["text.latex.preamble"] = r"\usepackage{bm}"

    dataset_map = {
        "inshop": "InShop",
        "sop": "SOP",
        "pitts": "Pittsburgh30K",
        "tokyo_val": "Tokyo-Val",
    }

    # Lists to store legend handles and labels
    handles = []
    labels = []

    # Loop through each dataset
    for i in range(2):
        for j in range(2):
            idx = i * 2 + j
            dataset_name = dataset_names[idx]

            # open pickle file
            all_results = []
            for embedding in embedding_list:
                with open(
                    f"data/pkl/{dataset_name}_pca_ablations_{metric_name}@{k}_{embedding}.pkl",
                    "rb",
                ) as f:
                    results = pickle.load(f)
                    all_results.append(results)

            lables = {
                "pca": "PCA",
                "pca_norm": "VA-PCA (Ours)",
                "pca_norm_random": "VA-PCA + Random",
                "norm_random": "Norm + Random",
            }

            metric_font = {
                "recall": "Recall",
                "map": "mAP",
            }

            # different colors for each embedding
            colors_list = {
                'clip': 'red',
                'vit': 'blue',
                'dinob16': 'green',
                'dinov2_large': 'orange',
            }

            for idx in range(len(embedding_list)):
                embedding = embedding_list[idx]
                results = all_results[idx]
                for key in results:
                    if key == "pca":
                        axs[i, j].plot(results["comp_range"], results[key], label=f'{embedding} with {lables[key]}', color=colors_list[embedding], linestyle="--")
                    elif key == "pca_norm":
                        axs[i, j].plot(results["comp_range"], results[key], label=f'{embedding} with {lables[key]}', color=colors_list[embedding], linestyle="-")

            axs[i, j].set_title(dataset_map[dataset_name], fontsize=20)
            axs[i, j].set_xlim(0, 800)
            axs[i, j].set_xlabel("Embedding Dimension", fontsize=18)
            axs[i, j].set_ylabel(f"{metric_font[metric_name]}@{k}", fontsize=18)

            # # Add handles and labels for legends
            if i==0 and j== 0:
                handles_axs, labels_axs = axs[i, j].get_legend_handles_labels()
                handles.extend(handles_axs)
                labels.extend(labels_axs)

    # Common legend and title
    fig.legend(handles, labels, loc='lower center', bbox_to_anchor=(0.5, -0.1), shadow=True, ncol=3, fontsize=18)
    fig.suptitle("Impact of VA-PCA on Retrieval Performance Across Four Datasets", fontsize=25)

    # Save plot
    plt.savefig(f"data/plots/vapca_{metric_name}@{k}_combined.png", bbox_inches='tight')
    logger.info(f"Saved plot: data/plots/vapca_{metric_name}@{k}_combined.png")
    plt.clf()



def theoretical_analysis(k, embedding, dataset_names, metric_name):
    set_plot_properties()
    fig, axs = plt.subplots(1, 2, figsize=(10, 4))

    plt.rcParams["text.usetex"] = True
    plt.rcParams["text.latex.preamble"] = r"\usepackage{bm}"

    # Loop through each dataset name
    leg_handles = []
    leg_labels = []
    for idx, dataset_name in enumerate(dataset_names):
        # open pickle file
        with open(
            f"data/pkl/{dataset_name}_pca_ablations_{metric_name}@{k}_{embedding}.pkl",
            "rb",
        ) as f:
            results = pickle.load(f)

        labels = {
            "pca": "PCA",
            "pca_norm": "VA-PCA (Ours)",
            "pca_norm_random": "VA-PCA-RS",
            "norm_random": "S-RS",
        }

        dataset_map ={
            "inshop": "InShop",
            "pitts": "Pittsburgh30K",
            "sop": "SOP",
            "tokyo_val": "Tokyo-Val",
        }

        metric_font = {
            "recall": "Recall",
            "map": "mAP",
        }

        
        for key in results:
            if key == "comp_range":
                continue
            axs[idx].plot(results["comp_range"], results[key], label=labels[key])


        axs[idx].set_title(
            f"Ablations with Dino on {dataset_map[dataset_name]}",
            fontsize=20,
        )

        axs[idx].set_xlabel("Embedding Dimension", fontsize=18)
        axs[idx].set_ylabel(f"{metric_font[metric_name]}@{k}", fontsize=18)

        # # Add handles and labels for legends
        if idx==0:
            handles_axs, labels_axs = axs[idx].get_legend_handles_labels()
            leg_handles.extend(handles_axs)
            leg_labels.extend(labels_axs)

    # Add a common legend at the bottom
    # plt.legend(loc="lower center", bbox_to_anchor=(0, -0.25), ncol=4, fontsize=12)

    # Adjust layout
    fig.legend(leg_handles, leg_labels, loc='lower center', bbox_to_anchor=(0.5, -0.1), shadow=True, ncol=4, fontsize=18)
    # plt.tight_layout()

    # Save the figure
    plt.savefig(f"data/plots/th_analysis_comparison_{metric_name}@{k}_{embedding}.png", bbox_inches='tight')
    plt.clf()

# Example usage:
# theoretical_analysis(k=10, embedding='embedding_type', dataset_names=['inshop', 'pitts'], metric_name='recall')

def plot_ss2d(combinations):
    '''
    combination examples
    combination = [
        ['inshop', 'dinov2_large'],
        ['pitts', 'dinos8']
    ]
    '''
    # import ss2d.yaml yaml data file
    import yaml
    with open('data/ss2d.yaml') as file:
        alldata = yaml.load(file, Loader=yaml.FullLoader)


    set_plot_properties()
    fig, axs = plt.subplots(1, 2, figsize=(10, 4))  # Create subplots side by side

    plt.rcParams["text.usetex"] = True
    plt.rcParams["text.latex.preamble"] = r"\usepackage{bm}"

    dataset_map = {
        "inshop": "InShop",
        "sop": "SOP",
        "pitts": "Pittsburgh30K",
        "tokyo_val": "TokyoVal",
    }

    approaches_map = {
        "pca": "PCA",
        "ssd": "SSD (Upper Bound)",
        "ss2d": "(SS)2D (Ours)",
        "vapca": "VA-PCA (Ours)",
        "vae": "VAE",
    }

    color_map = {
        "pca": "blue",
        "ssd": "red",
        "ss2d": "green",
        "vapca": "orange",
        "vae": "black",
    }

    embedding_map = {
        "clip": "CLIP",
        "vit": "ViT",
        "dinob16": "DINO-B16",
        "dinov2_large": "DINO-V2-Large",
        "dinos8": "DINO-S8",
    }

    # Lists to store legend handles and labels
    handles = []
    labels = []
    
    for i, (dataset_name, embedding) in enumerate(combinations):

        data = alldata[f'{dataset_name}_{embedding}']

        for key in data:
            methoddata = data[key]
            comp_range = methoddata.keys()
            mAP = methoddata.values()
            if key == 'vae' or key =='ssd':
                # scatter plot
                axs[i].scatter(comp_range, mAP, label=f'{approaches_map[key]}', color=color_map[key], marker='x')

            else:
                axs[i].plot(comp_range, mAP, label=f'{approaches_map[key]}', color=color_map[key])

        # font style bold in title
        axs[i].set_title(f'{dataset_map[dataset_name]} with {embedding_map[embedding]}', fontsize=20, fontproperties=FontProperties(weight='bold'))
        axs[i].set_xlabel("Embedding Dimension", fontsize=18)
        axs[i].set_ylabel(f"mAP@4", fontsize=18)

        # # Add handles and labels for legends
        if i == 0:
            handles_axs, labels_axs = axs[i].get_legend_handles_labels()
            handles.extend(handles_axs)
            labels.extend(labels_axs)
          # font style bold in title


    fig.legend(handles, labels, loc='lower center', bbox_to_anchor=(0.5, -0.2), shadow=True, ncol=5, fontsize=18)
    # fig.suptitle("Impact of VA-PCA on Retrieval Performance Across Four Datasets", fontsize=25)

    # Save plot
    plt.savefig(f"data/plots/ss2d_combined_line.png", bbox_inches='tight')
    logger.info(f"data/plots/ss2d_combined_line.png")
    plt.clf()

def plot_cosplace_vs_dinob16():
    set_plot_properties()
    plt.figure(figsize=(6, 4))

    # Create subplots side by side

    plt.rcParams["text.usetex"] = True
    plt.rcParams["text.latex.preamble"] = r"\usepackage{bm}"

    dataset_map = {
        "inshop": "InShop",
        "sop": "SOP",
        "pitts": "Pittsburgh30K",
        "tokyo_val": "TokyoVal",
    }

    embedding_map = {
        "clip": "CLIP",
        "vit": "ViT",
        "dinob16": "DINO-B16",
        "dinov2_large": "DINO-V2-Large",
        "dinos8": "DINO-S8",
    }

    approaches_map = {
        "pca": "PCA",
        "ssd": "SSD (Upper Bound)",
        "ss2d": "(SS)2D (Ours)",
        "vapca": "VA-PCA (Ours)",
        "pca_norm": "VA-PCA (Ours)",
        "vae": "VAE",
    }

    with open(
        f"data/pkl/pitts_pca_ablations_map@4_dinob16.pkl",
        "rb",
    ) as f:
        results_dinob16 = pickle.load(f)

    with open(
        f"data/pkl/pitts_pca_ablations_map@4_cosplace.pkl",
        "rb",
    ) as f:
        results_cosplace = pickle.load(f)
    
    for key in results_dinob16:
        if key == 'pca':
            plt.plot(results_dinob16["comp_range"], results_dinob16[key], label=f'Dino with {approaches_map[key]}', color='green', linestyle='--')
        elif key == 'pca_norm':
            plt.plot(results_dinob16["comp_range"], results_dinob16[key], label=f'Dino with {approaches_map[key]}', color='green',)

    for key in results_cosplace:
        if key == 'pca':
            plt.plot(results_cosplace["comp_range"], results_cosplace[key], label=f'Coslace with {approaches_map[key]}', color='red', linestyle='--')
        elif key == 'pca_norm':
            plt.plot(results_cosplace["comp_range"], results_cosplace[key], label=f'Cosplace with {approaches_map[key]}', color='red')

    plt.legend(loc="lower right", fontsize=15)

    # plt.title(
    #     f"PCA Ablations with DINO-B16 and CosPlace on Pittsburgh30K",
    #     fontsize=20,
    # )

    # set figure size

    plt.xlim(0, 800)
    plt.xlabel("Embedding Dimension", fontsize=20)
    plt.ylabel(f"mAP@4", fontsize=20)
    plt.savefig(f"data/plots/pitts_pca_ablations_map@4_dinob16_cosplace.png", bbox_inches='tight')


def test_plotting():
    # Example PMF data
    pmf1 = np.array([0.1, 0.2, 0.3, 0.4])
    pmf2 = np.array([0.3, 0.3, 0.2, 0.2])
    x = np.arange(len(pmf1))

    # Plot the first PMF
    plt.bar(x - 0.2, pmf1, width=0.4, color='blue', label='PMF 1', edgecolor='black', linewidth=1.2)

    # Plot the second PMF with a dashed line
    plt.bar(x + 0.2, pmf2, width=0.4, color='blue', label='PMF 2', edgecolor='black', linewidth=1.2, hatch='//')

    # Add labels and title
    plt.xlabel('X')
    plt.ylabel('Probability')
    plt.title('Histogram of Two PMFs')
    plt.legend()

    # Save the plot
    plt.savefig('data/plots/zz_pmf_plot.png')


if __name__ == "__main__":
    # plot_reduced_emb_exp_mem(k=3)
    # plot_reduced_emb_exp(k=3)
    # radar_plot()
    # plot_CCA(k=4, student_embedding="dinov2_small", teacher_embedding="dinob8")
    # boxplot()
    # plot_compute_vs_performance()
    # plot_pca_ablations_with_M(k=4, embedding="dinos8", dataset_name="pitts", metric_name="map")
    # plot_pca_ablations(k=4, embedding="dinos8", dataset_name="pitts", metric_name="map")
    # plot_flops_vs_performance()
    # plot_flops_vs_performance_full(k=4, embedding="dinos8", dataset_name="pitts", metric_name="map")
    # plot_va_pca_combined_block(k=4, embedding_list=["vit", "dinob16", "dinov2_large"], dataset_names=["inshop", "sop", "pitts", "tokyo_val"], metric_name="map")
    # plot_va_pca_combined_line(k=4, embedding_list=[ "vit", "dinob16", "dinov2_large"], dataset_names=["inshop", "sop", "pitts", "tokyo_val"], metric_name="map")
    theoretical_analysis(k=4, embedding="dinos8", dataset_names=["inshop", "pitts"], metric_name="map")
    # plot_ss2d(combinations=[['inshop', 'dinov2_large'], ['pitts', 'dinos8']])
    # plot_cosplace_vs_dinob16()
    # test_plotting()