import hydra
from omegaconf import DictConfig
import logging

# Note: Singletons might not be the best way to handle this, but it's a simple way to get started.

#### Config singleton ####
class ConfigSingleton:
    instance: DictConfig = None


# The conf directory is relative to this file.
@hydra.main(config_path="conf", config_name="config", version_base="1.3")
def setup_cfg(cfg: DictConfig):
    if ConfigSingleton.instance is not None:
        return
    print('### Running setup config')
    ConfigSingleton.instance = cfg


def get_cfg() -> DictConfig:
    if not ConfigSingleton.instance:
        raise ValueError("Config is not initialized.")
    return ConfigSingleton.instance


#### Logger singleton ####
class LoggerSingleton:
    instance = None


def setup_logger():
    if LoggerSingleton.instance is not None:
        return
    log = logging.getLogger("base_logger")
    cfg = get_cfg()
    log.setLevel(cfg.logger.level)
    formatter = logging.Formatter(cfg.logger.format)
    console_handler = logging.StreamHandler()
    console_handler.setFormatter(formatter)
    log.addHandler(console_handler)
    log.propagate = False

    LoggerSingleton.instance = log


def get_logger():
    if not LoggerSingleton.instance:
        raise ValueError("Logger is not initialized.")
    return LoggerSingleton.instance



setup_cfg()
setup_logger()
