import os
import pandas
import pickle
from utils.get_embeddings import dinov2, clip_imgs

root_dir =  "/home/pl22767/Data/waterbird_landfraction50_forest2water2/"
save_dir = "/nas/tirr/waterbird/"
BATCH_SIZE = 64

if __name__ == "__main__":
    train_img_folder = root_dir + 'train/real/'
    train_img_files = [train_img_folder + f for f in os.listdir(train_img_folder) if f.endswith('.jpg')]
    train_test_metadata_csv = pandas.read_csv(root_dir + 'train_real_metadata.csv')
    train_test_metadata_csv['img_filename_no_path'] = train_test_metadata_csv['img_filename'].apply(lambda x: x.split('/')[-1].replace(".jpg", ""))
    gt_train_test = []
    for img_name in train_img_files:
        img_name = img_name.replace(".jpg", "").split('/')[-1]
        gt_train_test.append(train_test_metadata_csv[train_test_metadata_csv['img_filename_no_path'] == img_name]['y'].values[0])
    test_img_folder = root_dir + 'test/'
    test_img_files = [test_img_folder + f for f in os.listdir(test_img_folder) if f.endswith('.jpg')]
    Q_metadata_csv = pandas.read_csv(root_dir + 'test_metadata.csv')
    Q_metadata_csv['img_filename_no_path'] = Q_metadata_csv['img_filename'].apply(lambda x: x.split('/')[-1].replace(".jpg", ""))
    for img_name in test_img_files:
        img_name = img_name.replace(".jpg", "").split('/')[-1]
        gt_train_test.append(Q_metadata_csv[Q_metadata_csv['img_filename_no_path'] == img_name]['y'].values[0])
    train_test_img_files = train_img_files + test_img_files
    numtrain_test = len(train_test_img_files)

    val_img_folder = root_dir + 'val/'
    val_img_files = [val_img_folder + f for f in os.listdir(val_img_folder) if f.endswith('.jpg')]
    gt_val = []
    val_metadata_csv = pandas.read_csv(root_dir + 'val_metadata.csv')
    val_metadata_csv['img_filename_no_path'] = val_metadata_csv['img_filename'].apply(lambda x: x.split('/')[-1].replace(".jpg", ""))
    for img_name in val_img_files:
        img_name = img_name.replace(".jpg", "").split('/')[-1]
        gt_val.append(val_metadata_csv[val_metadata_csv['img_filename_no_path'] == img_name]['y'].values[0])
    numQ = len(val_img_files)

    print("numtrain_test:", numtrain_test)
    print("numQ:", numQ)

    train_test_img_path_and_gt = [(f, gt) for f, gt in zip(train_test_img_files, gt_train_test)]
    val_img_path_and_gt = [(f, gt) for f, gt in zip(val_img_files, gt_val)]

    # save gt as pickle
    with open(save_dir + 'data/waterbird_gt_train_test.pkl', 'wb') as f:
        pickle.dump(train_test_img_path_and_gt, f)
    with open(save_dir + 'data/waterbird_gt_val.pkl', 'wb') as f:
        pickle.dump(val_img_path_and_gt, f)


    # get train_test embeddings
    img_emb = dinov2(train_test_img_files, BATCH_SIZE) # batched np array
    print("DINO embeddings done")
    with open(save_dir + 'data/waterbird_img_emb_train_test_dino.pkl', 'wb') as f:
        pickle.dump(img_emb, f)
    print("DINO embeddings saved")

    img_emb = clip_imgs(train_test_img_files, BATCH_SIZE) # batched np array
    print("CLIP embeddings done")
    with open(save_dir + 'data/waterbird_img_emb_train_test_clip.pkl', 'wb') as f:
        pickle.dump(img_emb, f)
    print("CLIP embeddings saved")

    # print("Loading ViT")
    # img_emb = vit(train_test_img_files, BATCH_SIZE) # batched np array
    # print("ViT embeddings done")
    # with open(save_dir + 'data/waterbird_img_emb_train_test_vit.pkl', 'wb') as f:
    #     pickle.dump(img_emb, f)
    # print("ViT embeddings saved")

    # get val embeddings
    img_emb = dinov2(val_img_files, BATCH_SIZE) # batched np array
    print("DINO embeddings done")
    with open(save_dir + 'data/waterbird_img_emb_val_dino.pkl', 'wb') as f:
        pickle.dump(img_emb, f)
    print("DINO embeddings saved")

    img_emb = clip_imgs(val_img_files, BATCH_SIZE) # batched np array
    print("CLIP embeddings done")
    with open(save_dir + 'data/waterbird_img_emb_val_clip.pkl', 'wb') as f:
        pickle.dump(img_emb, f)
    print("CLIP embeddings saved")
    
    # print("Loading ViT")
    # img_emb = vit(val_img_files, BATCH_SIZE) # batched np array
    # print("ViT embeddings done")
    # with open(save_dir + 'data/waterbird_img_emb_val_vit.pkl', 'wb') as f:
    #     pickle.dump(img_emb, f)
    # print("ViT embeddings saved")


# CUDA_VISIBLE_DEVICES=0 python waterbird_img_emb.py